#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Haiti(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Haiti holidays.

    References:
        * <https://web.archive.org/web/20250214180331/https://www.wipo.int/wipolex/en/legislation/details/7837>
        * <https://web.archive.org/web/20241206114430/https://haitiwonderland.com/haiti/histoire/jours-feries-en-haiti/27>
        * <https://web.archive.org/web/20241226203305/https://www.haiti-reference.info/pages/plan/generalites/calendrier-dhaiti/>
    """

    country = "HT"
    supported_categories = (OPTIONAL, PUBLIC)
    default_language = "fr_HT"
    supported_languages = ("en_US", "es", "fr_HT", "ht")
    # 1987 Constitution.
    start_year = 1987

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # ARTICLE 275.1 : National Holidays.
        # Both Battle of Vertieres Day and Armed Forces Day are on Nov 18, declared discretely.

        # National Independence Day.
        self._add_holiday_jan_1(tr("Fête de l'Indépendance Nationale"))

        # Ancestry Day.
        self._add_holiday_jan_2(tr("Jour des Aïeux"))

        # Agriculture and Labor Day.
        self._add_labor_day(tr("Fête de l'Agriculture et du Travail"))

        # Flag Day and University Day.
        self._add_holiday_may_18(tr("Fête du Drapeau et de l'Université"))

        # Commemoration of the Battle of Vertieres.
        self._add_holiday_nov_18(tr("Commémoration de la Bataille de Vertières"))

        # Armed Forces Day.
        self._add_holiday_nov_18(tr("Jour des Forces Armées"))

        # Other Common Holidays.

        # New Year's Day.
        self._add_new_years_day(tr("Nouvel An"))

        # Carnival.
        self._add_carnival_sunday(tr("Carnaval"))

        # Shrove Monday.
        self._add_carnival_monday(tr("Lundi Gras"))

        # Fat Tuesday.
        self._add_carnival_tuesday(tr("Mardi Gras"))

        # Good Friday.
        self._add_good_friday(tr("Vendredi Saint"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Pâques"))

        # Corpus Christi.
        self._add_corpus_christi_day(tr("Fête-Dieu"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Assomption de Marie"))

        # Death of Dessalines.
        self._add_holiday_oct_17(tr("Mort de Dessalines"))

        # All Saints' Day.
        self._add_all_saints_day(tr("La Toussaint"))

        # Day of the Dead.
        self._add_all_souls_day(tr("Fête des Morts"))

        # Christmas Day.
        self._add_christmas_day(tr("Noël"))

    def _populate_optional_holidays(self):
        # Ash Wednesday.
        self._add_ash_wednesday(tr("Mercredi des Cendres"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Jeudi Saint"))

        # National Sovereignty Day.
        self._add_holiday_may_23(tr("Fête de la Souveraineté Nationale"))

        # Feast of Lady of Perpetual Help, Patroness of Haiti.
        self._add_holiday_jun_27(tr("Fête de Notre-Dame du Perpétuel Secours, patronne d'Haiti"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Ascension"))

        # Birth Anniversary of Jean-Jacques Dessalines.
        self._add_holiday_sep_20(tr("Anniversaire de Naissance de Jean-Jacques Dessalines"))

        # Discovery Day.
        self._add_holiday_dec_5(tr("Jour de la Découverte"))


class HT(Haiti):
    pass


class HTI(Haiti):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, APR, JUN, AUG, SEP, OCT, DEC
from holidays.constants import BANK, HALF_DAY, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    MON_TO_NEXT_TUE,
    SAT_TO_NEXT_MON,
    SUN_TO_NEXT_MON,
    SUN_TO_NEXT_TUE,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
    ALL_TO_NEXT_MON,
)


class Australia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Australia holidays.

    References:
          * <https://web.archive.org/web/20250414072303/https://www.qld.gov.au/recreation/travel/holidays>
          * [ACT Holidays Act 1958](https://web.archive.org/web/20250322061953/https://www.legislation.act.gov.au/a/1958-19/)
          * [ACT 2013-2023](https://web.archive.org/web/20240401072340/https://www.cmtedd.act.gov.au/archived-content/holidays/previous-years)
          * [ACT Holidays (Anzac Day 2026) Declaration 2025 (No 1)](https://web.archive.org/web/20260116225801/https://legislation.act.gov.au/View/ni/2025-650/current/html/2025-650.html)
          * [NSW Banks and Bank Holidays Act 1912](https://web.archive.org/web/20241107225523/https://legislation.nsw.gov.au/view/html/repealed/current/act-1912-043)
          * [NSW Public Holidays Act 2010](https://web.archive.org/web/20250316173922/https://legislation.nsw.gov.au/view/html/inforce/current/act-2010-115)
          * [NSW 2026-2027](https://web.archive.org/web/20260216073138/https://www.nsw.gov.au/about-nsw/public-holidays)
          * [NT Public Holidays Act 1981](https://web.archive.org/web/20250315072128/https://legislation.nt.gov.au/api/sitecore/Act/PDF?id=12145)
          * [QLD Holidays Act 1983](https://web.archive.org/web/20250404230918/https://www.legislation.qld.gov.au/view/html/inforce/current/act-1983-018)
          * [QLD 2013-2028](https://web.archive.org/web/20150703042947/http://www.qld.gov.au/recreation/travel/holidays/public/)
          * [SA Holidays Act 1910](https://web.archive.org/web/20250420203417/https://www.legislation.sa.gov.au/LZ/C/A/HOLIDAYS%20ACT%201910.aspx)
          * [SA Public Holidays Act 2023](https://web.archive.org/web/20250429092113/https://www.legislation.sa.gov.au/lz?path=/C/A/Public%20Holidays%20Act%202023)
          * [SA 2023-2024](https://web.archive.org/web/20250404084235/https://www.safework.sa.gov.au/resources/public-holidays)
          * [SA 2007-2021](https://web.archive.org/web/20240610084716/https://www.safework.sa.gov.au/__data/assets/pdf_file/0007/235474/Public-Holidays-since-2007.pdf)
          * [TAS Statutory Holidays Act 2000](https://web.archive.org/web/20250423095807/https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096)
          * [VIC Public Holidays Act 1993](https://web.archive.org/web/20250212090816/https://www.legislation.vic.gov.au/in-force/acts/public-holidays-act-1993/027)
          * [VIC Minister appointment](https://web.archive.org/web/20240328142238/https://www.gazette.vic.gov.au/gazette/Gazettes2015/GG2015S229.pdf)
          * [VIC 2018-2024](https://web.archive.org/web/20250422235530/https://business.vic.gov.au/business-information/public-holidays)
          * [WA Public and Bank Holidays Act 1972](https://web.archive.org/web/20241106220940/https://www.legislation.wa.gov.au/legislation/statutes.nsf/law_a639.html)
          * [WA 2019-2023](https://web.archive.org/web/20240805123535/https://www.commerce.wa.gov.au/labour-relations/previous-years-public-holiday-dates)
    """

    country = "AU"
    default_language = "en_AU"
    # %s (observed).
    observed_label = tr("%s (observed)")
    start_year = 1801
    # fmt: off
    subdivisions = (
        "ACT",  # Australian Capital Territory.
        "NSW",  # New South Wales.
        "NT",   # Northern Territory.
        "QLD",  # Queensland.
        "SA",   # South Australia.
        "TAS",  # Tasmania.
        "VIC",  # Victoria.
        "WA",   # Western Australia.
    )
    # fmt: on
    subdivisions_aliases = {
        "Australian Capital Territory": "ACT",
        "New South Wales": "NSW",
        "Northern Territory": "NT",
        "Queensland": "QLD",
        "South Australia": "SA",
        "Tasmania": "TAS",
        "Victoria": "VIC",
        "Western Australia": "WA",
    }
    supported_categories = (BANK, HALF_DAY, PUBLIC)
    supported_languages = ("en_AU", "en_US", "th")

    @property
    def sovereign_birthday(self) -> str:
        """Sovereign's birthday holiday name."""
        return (
            # King's Birthday.
            tr("King's Birthday")
            if 1902 <= self._year <= 1951 or self._year >= 2023
            # Queen's Birthday.
            else tr("Queen's Birthday")
        )

    @property
    def australia_day(self) -> str:
        """Australia Day holiday name."""
        return (
            # Australia Day.
            tr("Australia Day")
            if self._year >= 1935
            # Anniversary Day.
            else tr("Anniversary Day")
        )

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, AustraliaStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Sovereign's Birthday.
        if 1902 <= self._year <= 1935:
            if self._year >= 1912:
                self._add_holiday_jun_3(self.sovereign_birthday)  # George V
            else:
                self._add_holiday_nov_9(self.sovereign_birthday)  # Edward VII

        if self.subdiv:
            return None

        # Common holidays.

        # New Year's Day.
        self._add_new_years_day(tr("New Year's Day"))

        # Australia Day.
        if self._year >= 1935:
            self._add_holiday_jan_26(self.australia_day)

        if self._year >= 1921:
            # ANZAC Day.
            self._add_anzac_day(tr("ANZAC Day"))

        # Christmas Day.
        self._add_christmas_day(tr("Christmas Day"))

        # Boxing Day.
        self._add_christmas_day_two(tr("Boxing Day"))

    def _populate_subdiv_act_public_holidays(self):
        # New Year's Day.
        # 1959-1992: SUN - move to MON.
        # 1993-2014: SAT, SUN - move to MON.
        # from 2015: SAT, SUN - add MON.

        # New Year's Day.
        dt = self._add_new_years_day(tr("New Year's Day"))
        if self._year >= 2015:
            self._add_observed(dt)
        elif self._year >= 1959:
            self._move_holiday(
                dt,
                rule=SAT_SUN_TO_NEXT_MON if self._year >= 1993 else SUN_TO_NEXT_MON,
                show_observed_label=False,
            )

        # Australia Day.
        # 1959-1989: not MON - move to MON.
        # from 1990: SAT, SUN - move to MON.

        # Australia Day.
        if self._year >= 1888:
            dt = self._add_holiday_jan_26(self.australia_day)
            if self._year >= 1959:
                self._move_holiday(
                    dt,
                    rule=SAT_SUN_TO_NEXT_MON if self._year >= 1990 else ALL_TO_NEXT_MON,
                    show_observed_label=False,
                )

        if self._year >= 1913:
            # Canberra Day.
            name = tr("Canberra Day")
            if self._year <= 1958 or self._year == 2012:
                self._add_holiday_mar_12(name)
            elif self._year <= 2007:
                self._add_holiday_3rd_mon_of_mar(name)
            else:
                self._add_holiday_2nd_mon_of_mar(name)

        # ANZAC Day.
        # from 1959: SUN - move to MON.
        # in 2026: move to MON.

        if self._year >= 1921:
            # ANZAC Day.
            dt = self._add_anzac_day(tr("ANZAC Day"))
            if self._year == 2026:
                self._move_holiday(dt, rule=SAT_SUN_TO_NEXT_MON, show_observed_label=False)
            elif self._year >= 1959:
                self._move_holiday(dt, rule=SUN_TO_NEXT_MON, show_observed_label=False)

        # Easter Saturday.
        self._add_holy_saturday(tr("Easter Saturday"))

        if self._year >= 2016:
            # Easter Sunday.
            self._add_easter_sunday(tr("Easter Sunday"))

        if self._year >= 2018:
            # Reconciliation Day.
            self._add_holiday_1st_mon_from_may_27(tr("Reconciliation Day"))

        # Sovereign's Birthday.
        if self._year >= 1936:
            self._add_holiday_2nd_mon_of_jun(self.sovereign_birthday)

        # Family & Community Day.
        # (First Monday of the September/October school holidays;
        # moved to the second Monday if this falls on Labour day).

        if 2010 <= self._year <= 2017:
            fc_dates = {
                2010: (SEP, 26),
                2011: (OCT, 10),
                2012: (OCT, 8),
                2013: (SEP, 30),
                2014: (SEP, 29),
                2015: (SEP, 28),
                2016: (SEP, 26),
                2017: (SEP, 25),
            }
            # Family & Community Day.
            self._add_holiday(tr("Family & Community Day"), fc_dates[self._year])

        # Labor Day.
        self._add_holiday_1st_mon_of_oct(tr("Labour Day"))

        # Boxing Day.
        # 1958-1991: SUN - to MON, MON - to TUE.
        # 1992-2013: SAT - to MON, SUN - to TUE, MON - to TUE.
        # from 2014: SAT - add MON, SUN - add TUE.
        # (Placed before Christmas Day for proper observed calculation).

        # Boxing Day.
        dt = self._add_christmas_day_two(tr("Boxing Day"))
        if self._year >= 2014:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE)
        elif self._year >= 1958:
            self._move_holiday(
                dt,
                rule=(SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE)
                if self._year >= 1992
                else (SUN_TO_NEXT_MON + MON_TO_NEXT_TUE),
                show_observed_label=False,
            )

        # Christmas Day.
        # 1958-1991: SUN - to MON.
        # 1992-2013: SAT, SUN - to MON.
        # from 2014: SAT - add MON, SUN - add TUE.

        # Christmas Day.
        dt = self._add_christmas_day(tr("Christmas Day"))
        if self._year >= 2014:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE)
        elif self._year >= 1958:
            self._move_holiday(
                dt,
                rule=SAT_SUN_TO_NEXT_MON if self._year >= 1992 else SUN_TO_NEXT_MON,
                show_observed_label=False,
            )

    def _populate_subdiv_act_bank_holidays(self):
        # Bank Holiday.
        self._add_holiday_1st_mon_of_aug(tr("Bank Holiday"))

    def _populate_subdiv_nsw_public_holidays(self):
        # New Year's Day.
        # 1912-2010: SUN - add MON.
        # from 2011: SAT, SUN - add MON.

        self._add_observed(
            # New Year's Day.
            self._add_new_years_day(tr("New Year's Day")),
            rule=SAT_SUN_TO_NEXT_MON if self._year >= 2011 else SUN_TO_NEXT_MON,
        )

        # Australia Day.
        # 1912-2010: SUN - add MON.
        # from 2011: SAT, SUN - to MON.

        if self._year >= 1888:
            dt = self._add_holiday_jan_26(
                # Australia Day.
                tr("Australia Day")
                if self._year >= 1946
                # Anniversary Day.
                else tr("Anniversary Day")
            )
            if self._year >= 2011:
                self._move_holiday(dt, show_observed_label=False)
            else:
                self._add_observed(dt, rule=SUN_TO_NEXT_MON)

        # Easter Saturday.
        self._add_holy_saturday(tr("Easter Saturday"))

        if self._year >= 2011:
            # Easter Sunday.
            self._add_easter_sunday(tr("Easter Sunday"))

        # ANZAC Day.
        # 1912-2010: SUN - add MON.
        # from 2011: normal.
        # 2026-2027: SAT, SUN - add MON.

        if self._year >= 1921:
            # ANZAC Day.
            dt = self._add_anzac_day(tr("ANZAC Day"))
            if self._year <= 2010:
                self._add_observed(dt, rule=SUN_TO_NEXT_MON)
            elif 2026 <= self._year <= 2027:
                self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON)

        # Labor Day.
        self._add_holiday_1st_mon_of_oct(tr("Labour Day"))

        # Sovereign's Birthday.
        if self._year >= 1936:
            self._add_holiday_2nd_mon_of_jun(self.sovereign_birthday)

        if 1912 <= self._year <= 2010:
            # Bank Holiday.
            self._add_holiday_1st_mon_of_aug(tr("Bank Holiday"))

        # Christmas Day.
        # 1912-2010: SUN - add TUE.
        # from 2011: SAT - add MON, SUN - add TUE.

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE if self._year >= 2011 else SUN_TO_NEXT_TUE,
        )

        # Boxing Day.
        # 1912-2010: SUN - add MON.
        # from 2011: SAT - add MON, SUN - add TUE.

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE if self._year >= 2011 else SUN_TO_NEXT_MON,
        )

    def _populate_subdiv_nsw_bank_holidays(self):
        if self._year >= 2011:
            # Bank Holiday.
            self._add_holiday_1st_mon_of_aug(tr("Bank Holiday"))

    def _populate_subdiv_nt_public_holidays(self):
        # New Year's Day.
        # 1982-2016: SAT, SUN - move to MON.
        # from 2017: SAT, SUN - add MON.

        # New Year's Day.
        dt = self._add_new_years_day(tr("New Year's Day"))
        if self._year >= 2017:
            self._add_observed(dt)
        elif self._year >= 1982:
            self._move_holiday(dt, show_observed_label=False)

        # Australia Day.
        # from 1982: SAT, SUN - move to MON.

        # Australia Day.
        if self._year >= 1888:
            dt = self._add_holiday_jan_26(self.australia_day)
            if self._year >= 1982:
                self._move_holiday(dt, show_observed_label=False)

        # ANZAC Day.
        # from 1982: SUN - to MON.

        if self._year >= 1921:
            # ANZAC Day.
            dt = self._add_anzac_day(tr("ANZAC Day"))
            if self._year >= 1982:
                self._move_holiday(dt, rule=SUN_TO_NEXT_MON, show_observed_label=False)

        # Easter Saturday.
        self._add_holy_saturday(tr("Easter Saturday"))

        if self._year >= 2024:
            # Easter Sunday.
            self._add_easter_sunday(tr("Easter Sunday"))

        # May Day.
        self._add_holiday_1st_mon_of_may(tr("May Day"))

        # Sovereign's Birthday.
        if self._year >= 1936:
            self._add_holiday_2nd_mon_of_jun(self.sovereign_birthday)

        # Picnic Day.
        self._add_holiday_1st_mon_of_aug(tr("Picnic Day"))

        # Boxing Day.
        # 1981-2022: SAT - to MON, SUN - to TUE, MON - to TUE.
        # from 2023: SAT - add MON, SUN - add TUE, MON - add TUE.
        # (Placed before Christmas Day for proper observed calculation).

        # Boxing Day.
        dt = self._add_christmas_day_two(tr("Boxing Day"))
        if self._year >= 2023:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE)
        elif self._year >= 1981:
            self._move_holiday(
                dt, rule=SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE, show_observed_label=False
            )

        # Christmas Day.
        # 1981-2015: SAT, SUN - to MON.
        # from 2016: SAT, SUN - add MON.

        # Christmas Day.
        dt = self._add_christmas_day(tr("Christmas Day"))
        if self._year >= 2016:
            self._add_observed(dt)
        elif self._year >= 1981:
            self._move_holiday(dt, show_observed_label=False)

    def _populate_subdiv_nt_half_day_holidays(self):
        if self._year >= 2016:
            # %s (from 7pm).
            begin_time_label = self.tr("%s (from 7pm)")

            # Christmas Eve.
            self._add_christmas_eve(begin_time_label % self.tr("Christmas Eve"))

            # New Year's Eve.
            self._add_new_years_eve(begin_time_label % self.tr("New Year's Eve"))

    def _populate_subdiv_qld_public_holidays(self):
        # New Year's Day.
        # 1984-2011: SUN - move to MON.
        # from 2012: SAT, SUN - add MON.

        # New Year's Day.
        dt = self._add_new_years_day(tr("New Year's Day"))
        if self._year >= 2012:
            self._add_observed(dt)
        elif self._year >= 1984:
            self._move_holiday(dt, rule=SUN_TO_NEXT_MON, show_observed_label=False)

        # Australia Day.
        # 1984-1995: not MON - move to MON.
        # from 1996: SAT, SUN - move to MON.

        # Australia Day.
        if self._year >= 1888:
            dt = self._add_holiday_jan_26(self.australia_day)
            if self._year >= 1984:
                self._move_holiday(
                    dt,
                    rule=SAT_SUN_TO_NEXT_MON if self._year >= 1996 else ALL_TO_NEXT_MON,
                    show_observed_label=False,
                )

        # ANZAC Day.
        # from 1984: SUN - move to MON.

        if self._year >= 1921:
            # ANZAC Day.
            dt = self._add_anzac_day(tr("ANZAC Day"))
            if self._year >= 1984:
                self._move_holiday(dt, rule=SUN_TO_NEXT_MON, show_observed_label=False)

        # Easter Saturday.
        self._add_holy_saturday(tr("Easter Saturday"))

        if self._year >= 2017:
            # Easter Sunday.
            self._add_easter_sunday(tr("Easter Sunday"))

        # Labor Day.
        name = tr("Labour Day")
        if 2013 <= self._year <= 2015:
            self._add_holiday_1st_mon_of_oct(name)
        else:
            self._add_holiday_1st_mon_of_may(name)

        # Sovereign's Birthday.
        if self._year >= 1936:
            if self._year <= 2015 and self._year != 2012:
                self._add_holiday_2nd_mon_of_jun(self.sovereign_birthday)
            else:
                self._add_holiday_1st_mon_of_oct(self.sovereign_birthday)

        # The Royal Queensland Show (Ekka).
        # The Show starts on the first Friday of August - providing this is
        # not prior to the 5th - in which case it will begin on the second
        # Friday. The Wednesday during the show is a public holiday.
        ekka_dates = {
            2020: (AUG, 14),
            2021: (OCT, 29),
        }
        # The Royal Queensland Show.
        name = tr("The Royal Queensland Show")
        if dt := ekka_dates.get(self._year):
            self._add_holiday(name, dt)
        else:
            # [1st FRI after Aug 5] + 5 days = [1st WED after Aug 10]
            self._add_holiday_1st_wed_from_aug_10(name)

        # Boxing Day.
        # 1984-2010: SUN - to MON, MON - to TUE.
        # from 2011: SAT - add MON, SUN - add TUE.
        # (Placed before Christmas Day for proper observed calculation).

        # Boxing Day.
        dt = self._add_christmas_day_two(tr("Boxing Day"))
        if self._year >= 2011:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE)
        elif self._year >= 1984:
            self._move_holiday(
                dt, rule=SUN_TO_NEXT_MON + MON_TO_NEXT_TUE, show_observed_label=False
            )

        # Christmas Day.
        # 1984-2010: SUN - to MON.
        # from 2011: SAT - add MON, SUN - add TUE.

        # Christmas Day.
        dt = self._add_christmas_day(tr("Christmas Day"))
        if self._year >= 2011:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE)
        elif self._year >= 1984:
            self._move_holiday(dt, rule=SUN_TO_NEXT_MON, show_observed_label=False)

    def _populate_subdiv_qld_half_day_holidays(self):
        if self._year >= 2019:
            # %s (from 6pm).
            begin_time_label = self.tr("%s (from 6pm)")

            # Christmas Eve.
            self._add_christmas_eve(begin_time_label % self.tr("Christmas Eve"))

    def _populate_subdiv_sa_public_holidays(self):
        # New Year's Day.
        # 1984-2003: SAT, SUN - move to MON.
        # 2004-2023: SAT - move to MON, SUN - add MON.
        # from 2024: SAT, SUN - add MON.

        # New Year's Day.
        dt = self._add_new_years_day(tr("New Year's Day"))
        if self._year >= 2024:
            self._add_observed(dt)
        elif self._year >= 2004:
            self._add_observed(dt, rule=SUN_TO_NEXT_MON)
            self._move_holiday(dt, rule=SAT_TO_NEXT_MON, show_observed_label=False)
        elif self._year >= 1984:
            self._move_holiday(dt, show_observed_label=False)

        # Australia Day.
        # 1984-1993: not MON - move to MON.
        # 1994-2003: SAT, SUN - move to MON.
        # 2004-2023: SAT - move to MON, SUN - add MON.
        # from 2024: SAT, SUN - move to MON.

        if self._year >= 1935:
            # Australia Day.
            dt = self._add_holiday_jan_26(self.australia_day)
            if 1994 <= self._year <= 2003 or self._year >= 2024:
                self._move_holiday(dt, show_observed_label=False)
            elif self._year >= 2004:
                self._add_observed(dt, rule=SUN_TO_NEXT_MON)
                self._move_holiday(dt, rule=SAT_TO_NEXT_MON, show_observed_label=False)
            elif self._year >= 1984:
                self._move_holiday(dt, rule=ALL_TO_NEXT_MON, show_observed_label=False)

        # Adelaide Cup Day.
        # First observed as Public Holidays in 1973: https://web.archive.org/web/20260122052707/https://www.racingsa.com.au/news/a-little-adelaide-cup-history
        # 2006-2023: changed each year by SA Government Proclamation from the 3rd Monday in May
        # to the 2nd Monday in March.
        # from 2024: changed to the 2nd Monday in March officially.

        # Adelaide Cup Day.
        name = tr("Adelaide Cup Day")
        if self._year >= 2006:
            self._add_holiday_2nd_mon_of_mar(name)
        elif self._year >= 1973:
            self._add_holiday_3rd_mon_of_may(name)

        # Easter Saturday.
        self._add_holy_saturday(tr("Easter Saturday"))

        if self._year >= 2024:
            # Easter Sunday.
            self._add_easter_sunday(tr("Easter Sunday"))

        if self._year >= 1921:
            # ANZAC Day.
            dt = self._add_anzac_day(tr("ANZAC Day"))
            if self._year <= 2023:
                self._add_observed(dt, rule=SUN_TO_NEXT_MON)

        # Sovereign's Birthday.
        if self._year >= 1936:
            self._add_holiday_2nd_mon_of_jun(self.sovereign_birthday)

        # Labor Day.
        self._add_holiday_1st_mon_of_oct(tr("Labour Day"))

        # Proclamation Day.
        # 1984-1992: SAT, SUN - move to MON.
        # 1993-2002: SAT - to MON, SUN - to TUE, MON - to TUE.
        # 2003-2023: SAT - to MON, SUN - add TUE, MON - add TUE.
        # from 2024: SAT - add MON, SUN - add TUE, MON - add TUE.
        # (Placed before Christmas Day for proper observed calculation).

        # Proclamation Day.
        dt = self._add_christmas_day_two(tr("Proclamation Day"))
        if self._year >= 2024:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE)
        elif self._year >= 2003:
            self._add_observed(dt, rule=SUN_TO_NEXT_TUE + MON_TO_NEXT_TUE)
            self._move_holiday(dt, rule=SAT_TO_NEXT_MON, show_observed_label=False)
        elif self._year >= 1993:
            self._move_holiday(
                dt, rule=SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE, show_observed_label=False
            )
        elif self._year >= 1984:
            self._move_holiday(dt, show_observed_label=False)

        # Christmas Day.
        # 1984-2002: SAT, SUN - move to MON.
        # 2003-2023: SAT - move to MON, SUN - add MON.
        # from 2024: SAT, SUN - add MON.

        # Christmas Day.
        dt = self._add_christmas_day(tr("Christmas Day"))
        if self._year >= 2024:
            self._add_observed(dt)
        elif self._year >= 2003:
            self._add_observed(dt, rule=SUN_TO_NEXT_MON)
            self._move_holiday(dt, rule=SAT_TO_NEXT_MON, show_observed_label=False)
        elif self._year >= 1984:
            self._move_holiday(dt, show_observed_label=False)

    def _populate_subdiv_sa_half_day_holidays(self):
        if self._year >= 2012:
            # %s (from 7pm).
            begin_time_label = self.tr("%s (from 7pm)")

            # Christmas Eve.
            self._add_christmas_eve(begin_time_label % self.tr("Christmas Eve"))

            # New Year's Eve.
            self._add_new_years_eve(begin_time_label % self.tr("New Year's Eve"))

    def _populate_subdiv_tas_public_holidays(self):
        # New Year's Day.
        # from 2001: SAT, SUN - move to MON.

        # New Year's Day.
        dt = self._add_new_years_day(tr("New Year's Day"))
        if self._year >= 2001:
            self._move_holiday(dt, show_observed_label=False)

        # Australia Day.
        # from 2001: SAT, SUN - move to MON.

        # Australia Day.
        if self._year >= 1888:
            dt = self._add_holiday_jan_26(self.australia_day)
            if self._year >= 2001:
                self._move_holiday(dt, show_observed_label=False)

        # Eight Hours Day.
        self._add_holiday_2nd_mon_of_mar(tr("Eight Hours Day"))

        if self._year <= 2010:
            # Easter Tuesday.
            self._add_easter_tuesday(tr("Easter Tuesday"))

        if self._year >= 1921:
            # ANZAC Day.
            self._add_anzac_day(tr("ANZAC Day"))

        # Sovereign's Birthday.
        if self._year >= 1936:
            self._add_holiday_2nd_mon_of_jun(self.sovereign_birthday)

        # Christmas Day.
        # 2000-2009: SAT - to MON, SUN - to TUE.
        # from 2010: SAT - add MON, SUN - add TUE.

        # Christmas Day.
        dt = self._add_christmas_day(tr("Christmas Day"))
        if self._year >= 2010:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE)
        elif self._year >= 2000:
            self._move_holiday(dt, rule=SAT_SUN_TO_NEXT_MON_TUE, show_observed_label=False)

        # Boxing Day.
        # from 2000: SAT - to MON, SUN - to TUE.

        # Boxing Day.
        dt = self._add_christmas_day_two(tr("Boxing Day"))
        if self._year >= 2000:
            self._move_holiday(dt, rule=SAT_SUN_TO_NEXT_MON_TUE, show_observed_label=False)

    def _populate_subdiv_vic_public_holidays(self):
        # New Year's Day.
        # 1994-1997: SUN - add MON.
        # 1998-2008: SUN - move to MON.
        # from 2009: SAT, SUN - add MON.

        # New Year's Day.
        dt = self._add_new_years_day(tr("New Year's Day"))
        if self._year >= 2009:
            self._add_observed(dt)
        elif self._year >= 1998:
            self._move_holiday(dt, rule=SUN_TO_NEXT_MON, show_observed_label=False)
        elif self._year >= 1994:
            self._add_observed(dt, rule=SUN_TO_NEXT_MON)

        # Australia Day.
        # from 2009: SAT, SUN - move to MON.

        # Australia Day.
        if self._year >= 1888:
            dt = self._add_holiday_jan_26(self.australia_day)
            if self._year >= 2009:
                self._move_holiday(dt, show_observed_label=False)

        # Labor Day.
        self._add_holiday_2nd_mon_of_mar(tr("Labour Day"))

        if self._year >= 2003:
            # Easter Saturday.
            self._add_holy_saturday(tr("Easter Saturday"))

        if self._year >= 2016:
            # Easter Sunday.
            self._add_easter_sunday(tr("Easter Sunday"))

        if self._year >= 1921:
            # ANZAC Day.
            self._add_anzac_day(tr("ANZAC Day"))

        # Sovereign's Birthday.
        if self._year >= 1936:
            self._add_holiday_2nd_mon_of_jun(self.sovereign_birthday)

        if self._year >= 2015:
            grand_final_dates = {
                2015: (OCT, 2),
                2016: (SEP, 30),
                # Rescheduled due to COVID-19.
                2020: (OCT, 23),
            }
            # Grand Final Day.
            name = tr("Grand Final Day")
            if dt := grand_final_dates.get(self._year):
                self._add_holiday(name, dt)
            else:
                self._add_holiday_1_day_prior_last_sat_of_sep(name)

        if self._year >= 2009:
            # Melbourne Cup Day.
            self._add_holiday_1st_tue_of_nov(tr("Melbourne Cup Day"))

        # Christmas Day.
        # 2008-2018: SAT - to MON, SUN - to TUE.
        # from 2019: SAT - add MON, SUN - add TUE.

        # Christmas Day.
        dt = self._add_christmas_day(tr("Christmas Day"))
        if self._year >= 2019:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE)
        elif self._year >= 2008:
            self._move_holiday(dt, rule=SAT_SUN_TO_NEXT_MON_TUE, show_observed_label=False)

        # Boxing Day.
        # 1994-2007: SUN - to MON.
        # from 2008: SAT - add MON, SUN - add TUE.

        # Boxing Day.
        dt = self._add_christmas_day_two(tr("Boxing Day"))
        if self._year >= 2008:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON_TUE)
        elif self._year >= 1994:
            self._move_holiday(dt, rule=SUN_TO_NEXT_MON, show_observed_label=False)

    def _populate_subdiv_wa_public_holidays(self):
        # New Year's Day.
        # from 1973: SAT, SUN - add MON.

        # New Year's Day.
        dt = self._add_new_years_day(tr("New Year's Day"))
        if self._year >= 1973:
            self._add_observed(dt)

        # Australia Day.
        # 1973-1993: not MON - move to MON.
        # from 1994: SAT, SUN - move to MON.

        # Australia Day.
        if self._year >= 1888:
            dt = self._add_holiday_jan_26(self.australia_day)
            if self._year >= 1994:
                self._move_holiday(dt, show_observed_label=False)
            elif self._year >= 1973:
                self._move_holiday(dt, rule=ALL_TO_NEXT_MON, show_observed_label=False)

        # Labor Day.
        self._add_holiday_1st_mon_of_mar(tr("Labour Day"))

        if self._year >= 2022:
            # Easter Sunday.
            self._add_easter_sunday(tr("Easter Sunday"))

        if self._year >= 1921:
            # ANZAC Day.
            dt = self._add_anzac_day(tr("ANZAC Day"))
            if self._year >= 1973:
                self._add_observed(dt)

        if self._year >= 1833:
            self._add_holiday_1st_mon_of_jun(
                # Western Australia Day.
                tr("Western Australia Day")
                if self._year >= 2012
                # Foundation Day.
                else tr("Foundation Day")
            )

        # Sovereign's Birthday.
        if self._year >= 1936:
            if self._year >= 1984:
                # Celebration Day for the Anniversary of the Birthday of the Reigning Sovereign
                # to be appointed for each year by proclamation published in the Government Gazette
                sovereign_birthday_dates = {
                    2011: (OCT, 28),
                    2012: (OCT, 1),
                    2024: (SEP, 23),
                }
                if dt := sovereign_birthday_dates.get(self._year):
                    self._add_holiday(self.sovereign_birthday, dt)
                else:
                    self._add_holiday_last_mon_of_sep(self.sovereign_birthday)
            else:
                self._add_holiday_2nd_mon_of_oct(self.sovereign_birthday)

        # Boxing Day.
        # 1972-1975: SAT - add MON, SUN - add TUE.
        # from 1976: SAT - add MON, SUN - add TUE, MON - add TUE.
        # (Placed before Christmas Day for proper observed calculation).

        # Boxing Day.
        dt = self._add_christmas_day_two(tr("Boxing Day"))
        if self._year >= 1972:
            self._add_observed(
                dt,
                rule=SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE
                if self._year >= 1976
                else SAT_SUN_TO_NEXT_MON_TUE,
            )

        # Christmas Day.
        # from 1972: SAT, SUN - add MON.

        # Christmas Day.
        dt = self._add_christmas_day(tr("Christmas Day"))
        if self._year >= 1972:
            self._add_observed(dt)


class AU(Australia):
    pass


class AUS(Australia):
    pass


class AustraliaStaticHolidays:
    # Special public holiday.
    special_public_holiday = tr("Special public holiday")

    special_public_holidays = {
        # National Day of Mourning for Queen Elizabeth II.
        2022: (SEP, 22, tr("National Day of Mourning for Queen Elizabeth II")),
    }

    special_act_public_holidays = {
        # Declared public holiday.
        2020: (APR, 20, tr("Declared public holiday")),
        # Additional public holiday.
        2021: (APR, 25, tr("Additional public holiday")),
    }

    special_qld_public_holidays = {
        # Queen's Diamond Jubilee.
        2012: (JUN, 11, tr("Queen's Diamond Jubilee")),
    }

    special_qld_public_holidays_observed = {
        # Christmas Day.
        2010: (DEC, 28, tr("Christmas Day")),
        # New Year's Day.
        2011: (JAN, 3, tr("New Year's Day")),
    }

    special_wa_public_holidays = {
        # In 2011 both ANZAC Day and Easter Monday fell on Monday 25 April.
        2011: (APR, 26, special_public_holiday),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars.gregorian import OCT
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, ALL_TO_NEAREST_MON_LATAM


class Guatemala(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Guatemala holidays.

    References:
        * <https://web.archive.org/web/20240419130706/http://www.bvnsa.com.gt/bvnsa/calendario_dias_festivos.php>
        * <https://web.archive.org/web/20250426043930/https://www.minfin.gob.gt/images/downloads/leyes_acuerdos/decretocong19_101018.pdf>

    Moving holidays:
        * [Decree 19-2018 start 18 oct 2018](https://web.archive.org/web/20250426043930/https://www.minfin.gob.gt/images/downloads/leyes_acuerdos/decretocong19_101018.pdf)
        * [Case 5536-2018 (CC) start 17 abr 2020](https://web.archive.org/web/20240625093244/https://leyes.infile.com/index.php?id=182&id_publicacion=81055)
    """

    country = "GT"
    default_language = "es"
    supported_languages = ("en_US", "es")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", ALL_TO_NEAREST_MON_LATAM)
        super().__init__(*args, **kwargs)

    def _is_observed(self, dt: date) -> bool:
        return dt >= date(2018, OCT, 18)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Año Nuevo"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Jueves Santo"))

        # Good Friday.
        self._add_good_friday(tr("Viernes Santo"))

        # Holy Saturday.
        self._add_holy_saturday(tr("Sábado Santo"))

        # Labor Day.
        dt = self._add_labor_day(tr("Día del Trabajo"))
        if self._year == 2019:
            self._move_holiday(dt)

        # Army Day.
        self._move_holiday(self._add_holiday_jun_30(tr("Día del Ejército")))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Día de la Asunción"))

        # Independence Day.
        self._add_holiday_sep_15(tr("Día de la Independencia"))

        # Revolution Day.
        dt = self._add_holiday_oct_20(tr("Día de la Revolución"))
        if self._year in {2018, 2019}:
            self._move_holiday(dt)

        # All Saints' Day.
        self._add_all_saints_day(tr("Día de Todos los Santos"))

        # Christmas Day.
        self._add_christmas_day(tr("Día de Navidad"))


class GT(Guatemala):
    pass


class GUA(Guatemala):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.calendars.gregorian import _timedelta


class _Mandaean:
    """
    Mandaean calendar for 1901-2100 years.
    https://en.wikipedia.org/wiki/Mandaean_calendar
    """

    # Begin of 445271 Mandaean year.
    START_DATE = date(1901, 8, 16)
    START_YEAR = 1901
    END_YEAR = 2100

    def new_year_date(self, year: int) -> date | None:
        """
        Return Gregorian date of Mandaean new year (1 Dowla) in a given Gregorian year.
        """
        if year < _Mandaean.START_YEAR or year > _Mandaean.END_YEAR:
            return None

        return _timedelta(_Mandaean.START_DATE, 365 * (year - _Mandaean.START_YEAR))

    def mandaean_to_gregorian(self, year: int, month: int, day: int) -> date | None:
        """
        Return Gregorian date of Mandaean day and month of the year that begins in a given
        Gregorian year.
        Extra 5 days inserted after 8th month are considered as 13th month.
        """
        start_date = self.new_year_date(year)
        if not start_date:
            return None

        if not (1 <= month <= 13) or not (1 <= day <= 30) or (month == 13 and not (1 <= day <= 5)):
            return None

        if month < 9:
            delta = 30 * (month - 1)
        elif month == 13:
            delta = 30 * 8
        else:
            delta = 30 * (month - 1) + 5

        return _timedelta(start_date, delta + day - 1)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

ETHIOPIAN_CALENDAR = "ETHIOPIAN_CALENDAR"


def is_ethiopian_leap_year(year: int) -> bool:
    """Determine if the Ethiopian year that starts in the given Gregorian year is a leap year.

    Ethiopian leap years follow the Coptic/Julian rule:
        * Every 4 years without exception (no century rule).
        * The Ethiopian year starts on September 11 (or 12 in an Ethiopian leap year).

    Args:
        year:
            Gregorian year to evaluate.

    Returns:
        `True` if an Ethiopian leap year starts in the given Gregorian year, `False` otherwise.
    """

    return (year + 1) % 4 == 0



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, SEP, OCT, DEC
from holidays.constants import HALF_DAY, PUBLIC
from holidays.groups import ChineseCalendarHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_SUN_TO_NEXT_WORKDAY


class China(ObservedHolidayBase, ChineseCalendarHolidays, InternationalHolidays, StaticHolidays):
    """China holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_China>
        * [Festivals and Public Holidays](https://zh.wikipedia.org/wiki/中华人民共和国节日与公众假期)
        * [2026](https://web.archive.org/web/20251111205945/https://www.gov.cn/zhengce/content/202511/content_7047090.htm)
        * [2025](https://web.archive.org/web/20250424041657/https://www.gov.cn/zhengce/content/202411/content_6986382.htm)
        * [2024 changes (Order #795)](https://web.archive.org/web/20250228151847/https://www.gov.cn/zhengce/content/202411/content_6986380.htm)
        * [2024](https://web.archive.org/web/20250227033646/https://www.gov.cn/zhengce/content/202310/content_6911527.htm)
        * [2023](https://web.archive.org/web/20250414125053/https://www.gov.cn/gongbao/content/2023/content_5736714.htm)
        * [2022](https://web.archive.org/web/20250413071341/http://www.gov.cn/gongbao/content/2021/content_5651728.htm)
        * [2021](https://web.archive.org/web/20250424075325/https://www.gov.cn/gongbao/content/2020/content_5567750.htm)
        * [2020 Extensions](https://web.archive.org/web/20250427184932/https://www.gov.cn/zhengce/zhengceku/2020-01/27/content_5472352.htm)
        * [2020](https://web.archive.org/web/20241222150612/https://www.gov.cn/gongbao/content/2019/content_5459138.htm)
        * [2019 Extensions](https://web.archive.org/web/20250901174347/https://www.gov.cn/gongbao/content/2019/content_5380356.htm)
        * [2019](https://web.archive.org/web/20241202235023/https://www.gov.cn/gongbao/content/2018/content_5350046.htm)
        * [2018](https://web.archive.org/web/20231205013402/https://www.gov.cn/gongbao/content/2017/content_5248221.htm)
        * [2017](https://web.archive.org/web/20231205013333/https://www.gov.cn/gongbao/content/2016/content_5148793.htm)
        * [2016](https://web.archive.org/web/20231205013233/https://www.gov.cn/gongbao/content/2016/content_2979719.htm)
        * [2015](https://web.archive.org/web/20250427025304/https://www.gov.cn/gongbao/content/2015/content_2799019.htm)
        * [2014](https://web.archive.org/web/20250426080722/https://www.gov.cn/gongbao/content/2014/content_2561299.htm)
        * [2013](https://web.archive.org/web/20240229121946/https://www.gov.cn/gongbao/content/2012/content_2292057.htm)
        * [2012](https://web.archive.org/web/20250420220922/https://www.gov.cn/gongbao/content/2011/content_2020918.htm)
        * [2011](https://web.archive.org/web/20240812070712/https://www.gov.cn/gongbao/content/2010/content_1765282.htm)
        * [2010](https://web.archive.org/web/20101210083603/http://www.gov.cn:80/gongbao/content/2009/content_1487011.htm)
        * [2009](https://web.archive.org/web/20230726083438/https://www.gov.cn/gongbao/content/2008/content_1175823.htm)
        * [2008](https://web.archive.org/web/20240610103541/https://www.gov.cn/gongbao/content/2008/content_859870.htm)
        * [2007](https://web.archive.org/web/20230727050141/https://www.gov.cn/gongbao/content/2007/content_503397.htm)
        * [2006](https://web.archive.org/web/20221130214247/https://zh.wikisource.org/wiki/国务院办公厅关于2006年部分节假日安排的通知)
        * [2005](https://web.archive.org/web/20210212184000/https://zh.wikisource.org/wiki/国务院办公厅关于2005年部分节假日安排的通知)
        * [2004](https://web.archive.org/web/20210212183857/https://zh.wikisource.org/wiki/国务院办公厅关于2004年部分节假日安排的通知)
        * [2003](https://web.archive.org/web/20210302090553/https://zh.wikisource.org/wiki/国务院办公厅关于2003年部分节假日休息安排的通知)
        * [2002](https://web.archive.org/web/20180122201149/https://zh.wikisource.org/wiki/国务院办公厅关于2002年部分节假日休息安排的通知)
        * [2001](https://web.archive.org/web/20180123032517/https://zh.wikisource.org/wiki/国务院办公厅关于2001年春节、“五一”、“十一”放假安排的通知)

    Checked With:
        * <https://web.archive.org/web/20250213085558/https://www.officeholidays.com/countries/china/2023>
        * <https://web.archive.org/web/20250210121944/https://www.china-briefing.com/news/china-public-holiday-2023-schedule/>
        * <https://web.archive.org/web/20250218010025/https://www.timeanddate.com/calendar/?year=2023&country=41>
        * [2001-2010](https://web.archive.org/web/20250429074341/https://m.wannianli.tianqi.com/fangjiaanpai/2001.html)

    Limitations:
        * Only checked with the official General Office of the State Council Notice from 2001
            onwards.
        * Due to its complexity, need yearly checks 3-weeks before year's end each year.
    """

    country = "CN"
    # %s (estimated).
    estimated_label = tr("%s（推定）")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s（观察日，推定）")
    # %s (observed).
    observed_label = tr("%s（观察日）")
    supported_categories = (PUBLIC, HALF_DAY)
    default_language = "zh_CN"
    supported_languages = ("en_US", "th", "zh_CN", "zh_TW")
    # Proclamation of the People's Republic of China on Oct 1, 1949.
    start_year = 1950

    def __init__(self, *args, **kwargs):
        ChineseCalendarHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=ChinaStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        kwargs.setdefault("observed_since", 2000)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        dts_observed = set()

        # 元旦 (simp.) / 新年 (trad.)
        # Status: In-Use (Statutory).
        # Jan 1 in 1949, 1999, 2007, and 2013 revision.
        # Consecutive Holidays are available from 2002, except in 2014/2016/2017/2018.

        # New Year's Day.
        dts_observed.add(self._add_new_years_day(tr("元旦")))

        # 春节
        # Status: In-Use (Statutory).
        # Day 1-3 of Chinese New Year in 1949, 1999, 2007, and 2013 revision.
        # 2007 revision introduced New Year's Eve (农历除夕) instead of
        # New Year's 3rd day; 2013 revision returned it back;
        # 2024 revision returned also New Year's Eve.

        # Spring Festival Golden Weekend
        # Checked with Official Notice from 2001-2025.
        # Consecutive Holidays are available from 2000 (1999 rev.).

        # Chinese New Year (Spring Festival).
        chinese_new_year = tr("春节")
        # Chinese New Year's Eve.
        chinese_new_years_eve = tr("农历除夕")
        dts_observed.add(self._add_chinese_new_years_day(chinese_new_year))
        dts_observed.add(self._add_chinese_new_years_day_two(chinese_new_year))
        if 2008 <= self._year <= 2013:
            dts_observed.add(self._add_chinese_new_years_eve(chinese_new_years_eve))
        else:
            dts_observed.add(self._add_chinese_new_years_day_three(chinese_new_year))
        if self._year >= 2025:
            dts_observed.add(self._add_chinese_new_years_eve(chinese_new_years_eve))

        # 劳动节
        # Status: In-Use (Statutory).
        # May 1 in 1949, 1999, 2007, and 2013 revision.
        # Additional Holidays (May 2-3) are available from 2000 (1999 rev.) - 2007 (2007 rev.).
        # May 2 returned in 2024 revision.

        # Labor Day Golden Weekend
        # Checked with Official Notice from 2001-2025.
        # Consecutive Holidays are available from 2002, with exception of ????-????.

        # Labor Day.
        labor_day = tr("劳动节")
        dts_observed.add(self._add_labor_day(labor_day))
        if 2000 <= self._year <= 2007:
            dts_observed.add(self._add_labor_day_two(labor_day))
            dts_observed.add(self._add_labor_day_three(labor_day))
        elif self._year >= 2025:
            dts_observed.add(self._add_labor_day_two(labor_day))

        # 国庆节
        # Status: In-Use (Statutory).
        # Oct 1-2 in 1949, 1999, 2007, and 2013 revision
        # Additional Holiday (Oct 3) is available from Sep 1999 (1999 rev.).

        # National Day Golden Weekend
        # Checked with Official Notice from 2001-2023.

        # National Day.
        national_day = tr("国庆节")
        dts_observed.add(self._add_holiday_oct_1(national_day))
        dts_observed.add(self._add_holiday_oct_2(national_day))
        if self._year >= 1999:
            dts_observed.add(self._add_holiday_oct_3(national_day))

        if self._year >= 2008:
            # 清明节
            # Status: In-Use (Statutory).
            # Tomb-Sweeping Day in 2007, and 2013 revision.
            # Consecutive Holidays are available from 2008, except in 2014/2015/2016/2019.

            # Tomb-Sweeping Day.
            dts_observed.add(self._add_qingming_festival(tr("清明节")))

            # 端午节
            # Status: In-Use (Statutory).
            # Dragon Boat Festival in 2007, and 2013 revision.
            # Consecutive Holidays are available from 2008, except in 2014/2015/2018/2019/2023.

            # Dragon Boat Festival.
            dragon_boat_festival = self._add_dragon_boat_festival(tr("端午节"))
            if self._year != 2012:
                dts_observed.add(dragon_boat_festival)

            # 中秋节
            # Status: In-Use (Statutory).
            # Mid-Autumn Festival in 2007, and 2013 revision.
            # Consecutive Holidays are available from 2008, except in 2010/2014/2015/2018/2019.
            # Extra Day (Oct 8) is instead added to the National Day Week if overlaps.

            # Mid-Autumn Festival.
            mid_autumn_festival = self._add_mid_autumn_festival(tr("中秋节"))
            if self._year != 2015:
                dts_observed.add(mid_autumn_festival)

        if self.observed:
            self._populate_observed(dts_observed, multiple=True)

    def _populate_half_day_holidays(self):
        # No in lieus are given for this category.

        # International Women's Day.
        self._add_womens_day(tr("国际妇女节"))

        # Youth Day.
        self._add_holiday_may_4(tr("五四青年节"))

        # Children's Day.
        self._add_childrens_day(tr("六一儿童节"))

        # Army Day.
        self._add_holiday_aug_1(tr("建军节"))


class CN(China):
    pass


class CHN(China):
    pass


class ChinaStaticHolidays:
    # Date format (see strftime() Format Codes).
    substituted_date_format = tr("%Y-%m-%d")
    # Day off (substituted from %s).
    substituted_label = tr("休息日（%s日起取代）")

    # Chinese New Year (Spring Festival).
    chinese_new_year = tr("春节")

    # Chinese New Year (Spring Festival) Extended Holiday.
    chinese_new_year_extended = tr("春节延长假期")

    # Dragon Boat Festival.
    dragon_boat_festival = tr("端午节")

    # Mid-Autumn Festival.
    mid_autumn_festival = tr("中秋节")

    # 70th Anniversary of the Victory of the Chinese People's War of Resistance against
    # Japanese Aggression and the World Anti-Fascist War.
    victory_70_anniversary = tr("中国人民抗日战争暨世界反法西斯战争胜利70周年纪念日")

    special_public_holidays = {
        2001: (
            (JAN, 29, JAN, 20),  # Spring Festival
            (JAN, 30, JAN, 21),  # Spring Festival
            (MAY, 4, APR, 28),  # Labor Day
            (MAY, 7, APR, 29),  # Labor Day
            (OCT, 4, SEP, 29),  # National Day
            (OCT, 5, SEP, 30),  # National Day
        ),
        2002: (
            (JAN, 2, DEC, 29, 2001),  # New Year's Day
            (JAN, 3, DEC, 30, 2001),  # New Year's Day
            (FEB, 15, FEB, 9),  # Spring Festival
            (FEB, 18, FEB, 10),  # Spring Festival
            (MAY, 6, APR, 27),  # Labor Day
            (MAY, 7, APR, 28),  # Labor Day
            (OCT, 4, SEP, 28),  # National Day
            (OCT, 7, SEP, 29),  # National Day
        ),
        2003: (
            (FEB, 6, FEB, 8),  # Spring Festival
            (FEB, 7, FEB, 9),  # Spring Festival
            (MAY, 6, APR, 26),  # Labor Day
            (MAY, 7, APR, 27),  # Labor Day
            (OCT, 6, SEP, 27),  # National Day
            (OCT, 7, SEP, 28),  # National Day
        ),
        2004: (
            (JAN, 27, JAN, 17),  # Spring Festival
            (JAN, 28, JAN, 18),  # Spring Festival
            (MAY, 6, MAY, 8),  # Labor Day
            (MAY, 7, MAY, 9),  # Labor Day
            (OCT, 6, OCT, 9),  # National Day
            (OCT, 7, OCT, 10),  # National Day
        ),
        2005: (
            (FEB, 14, FEB, 5),  # Spring Festival
            (FEB, 15, FEB, 6),  # Spring Festival
            (MAY, 5, APR, 30),  # Labor Day
            (MAY, 6, MAY, 8),  # Labor Day
            (OCT, 6, OCT, 8),  # National Day
            (OCT, 7, OCT, 9),  # National Day
        ),
        2006: (
            (JAN, 3, DEC, 31, 2005),  # New Year's Day
            (FEB, 2, JAN, 28),  # Spring Festival
            (FEB, 3, FEB, 5),  # Spring Festival
            (MAY, 4, APR, 29),  # Labor Day
            (MAY, 5, APR, 30),  # Labor Day
            (OCT, 5, SEP, 30),  # National Day
            (OCT, 6, OCT, 8),  # National Day
        ),
        2007: (
            (JAN, 2, DEC, 30, 2006),  # New Year's Day
            (JAN, 3, DEC, 31, 2006),  # New Year's Day
            (FEB, 22, FEB, 17),  # Spring Festival
            (FEB, 23, FEB, 25),  # Spring Festival
            (MAY, 4, APR, 28),  # Labor Day
            (MAY, 7, APR, 29),  # Labor Day
            (OCT, 4, SEP, 29),  # National Day
            (OCT, 5, SEP, 30),  # National Day
            (DEC, 31, DEC, 29),  # New Year's Day
        ),
        2008: (
            (FEB, 11, FEB, 2),  # Spring Festival
            (FEB, 12, FEB, 3),  # Spring Festival
            (MAY, 2, MAY, 4),  # Labor Day
            (SEP, 29, SEP, 27),  # National Day
            (SEP, 30, SEP, 28),  # National Day
        ),
        2009: (
            (JAN, 2, JAN, 4),  # New Year's Day
            (JAN, 29, JAN, 24),  # Spring Festival
            (JAN, 30, FEB, 1),  # Spring Festival
            (MAY, 29, MAY, 31),  # Dragon Boat Festival
            (OCT, 7, SEP, 27),  # National Day
            (OCT, 8, OCT, 10),  # National Day
        ),
        2010: (
            (FEB, 18, FEB, 20),  # Spring Festival
            (FEB, 19, FEB, 21),  # Spring Festival
            (JUN, 14, JUN, 12),  # Dragon Boat Festival
            (JUN, 15, JUN, 13),  # Dragon Boat Festival
            (SEP, 23, SEP, 19),  # Mid-Autumn Festival
            (SEP, 24, SEP, 25),  # Mid-Autumn Festival
            (OCT, 6, SEP, 26),  # National Day
            (OCT, 7, OCT, 9),  # National Day
        ),
        2011: (
            (FEB, 7, JAN, 30),  # Spring Festival
            (FEB, 8, FEB, 12),  # Spring Festival
            (APR, 4, APR, 2),  # Tomb-Sweeping Day
            (OCT, 6, OCT, 8),  # National Day
            (OCT, 7, OCT, 9),  # National Day
        ),
        2012: (
            (JAN, 3, DEC, 31, 2011),  # New Year's Day
            (JAN, 26, JAN, 21),  # Spring Festival
            (JAN, 27, JAN, 29),  # Spring Festival
            (APR, 2, MAR, 31),  # Tomb-Sweeping Day
            (APR, 3, APR, 1),  # Tomb-Sweeping Day
            (APR, 30, APR, 28),  # Labor Day
            (OCT, 5, SEP, 29),  # National Day
        ),
        2013: (
            (JAN, 2, JAN, 5),  # New Year's Day
            (JAN, 3, JAN, 6),  # New Year's Day
            (FEB, 14, FEB, 16),  # Spring Festival
            (FEB, 15, FEB, 17),  # Spring Festival
            (APR, 5, APR, 7),  # Tomb-Sweeping Day
            (APR, 29, APR, 27),  # Labor Day
            (APR, 30, APR, 28),  # Labor Day
            (JUN, 10, JUN, 8),  # Dragon Boat Festival
            (JUN, 11, JUN, 9),  # Dragon Boat Festival
            (SEP, 20, SEP, 22),  # Mid-Autumn Festival
            (OCT, 4, SEP, 29),  # National Day
            (OCT, 7, OCT, 12),  # National Day
        ),
        2014: (
            (FEB, 5, JAN, 26),  # Spring Festival
            (FEB, 6, FEB, 8),  # Spring Festival
            (MAY, 2, MAY, 4),  # Labor Day
            (OCT, 6, SEP, 28),  # National Day
            (OCT, 7, OCT, 11),  # National Day
        ),
        2015: (
            (JAN, 2, JAN, 4),  # New Year's Day
            (FEB, 18, FEB, 15),  # Spring Festival
            (FEB, 24, FEB, 28),  # Spring Festival
            (SEP, 3, victory_70_anniversary),
            (SEP, 4, SEP, 6),  # 70th Anniversary of the Victory
            (OCT, 7, OCT, 10),  # National Day
        ),
        2016: (
            (FEB, 11, FEB, 6),  # Spring Festival
            (FEB, 12, FEB, 14),  # Spring Festival
            (JUN, 10, JUN, 12),  # Dragon Boat Festival
            (SEP, 16, SEP, 18),  # Mid-Autumn Festival
            (OCT, 6, OCT, 8),  # National Day
            (OCT, 7, OCT, 9),  # National Day
        ),
        2017: (
            (JAN, 27, JAN, 22),  # Spring Festival
            (FEB, 2, FEB, 4),  # Spring Festival
            (APR, 3, APR, 1),  # Tomb-Sweeping Day
            (MAY, 29, MAY, 27),  # Dragon Boat Festival
            (OCT, 6, SEP, 30),  # National Day
        ),
        2018: (
            (FEB, 15, FEB, 11),  # Spring Festival
            (FEB, 21, FEB, 24),  # Spring Festival
            (APR, 6, APR, 8),  # Tomb-Sweeping Day
            (APR, 30, APR, 28),  # Labor Day
            (OCT, 4, SEP, 29),  # National Day
            (OCT, 5, SEP, 30),  # National Day
            (DEC, 31, DEC, 29),  # New Year's Day
        ),
        2019: (
            (FEB, 4, FEB, 2),  # Spring Festival
            (FEB, 8, FEB, 3),  # Spring Festival
            (MAY, 2, APR, 28),  # Labor Day
            (MAY, 3, MAY, 5),  # Labor Day
            (OCT, 4, SEP, 29),  # National Day
            (OCT, 7, OCT, 12),  # National Day
        ),
        2020: (
            (JAN, 24, JAN, 19),  # Spring Festival
            # JAN, 30 in special_public_holidays_observed
            (JAN, 31, chinese_new_year_extended),  # Spring Festival Extended Holiday
            (FEB, 1, chinese_new_year_extended),  # Spring Festival Extended Holiday
            (FEB, 2, chinese_new_year_extended),  # Spring Festival Extended Holiday
            (MAY, 4, APR, 26),  # Labor Day
            (MAY, 5, MAY, 9),  # Labor Day
            (JUN, 26, JUN, 28),  # Dragon Boat Festival
            (OCT, 7, SEP, 27),  # National Day
            (OCT, 8, OCT, 10),  # National Day
        ),
        2021: (
            (FEB, 11, FEB, 7),  # Spring Festival
            (FEB, 17, FEB, 20),  # Spring Festival
            (MAY, 4, APR, 25),  # Labor Day
            (MAY, 5, MAY, 8),  # Labor Day
            (SEP, 20, SEP, 18),  # Mid-Autumn Festival
            (OCT, 6, SEP, 26),  # National Day
            (OCT, 7, OCT, 9),  # National Day
        ),
        2022: (
            (JAN, 31, JAN, 29),  # Spring Festival
            (FEB, 4, JAN, 30),  # Spring Festival
            (APR, 4, APR, 2),  # Tomb-Sweeping Day
            (MAY, 3, APR, 24),  # Labor Day
            (MAY, 4, MAY, 7),  # Labor Day
            (OCT, 6, OCT, 8),  # National Day
            (OCT, 7, OCT, 9),  # National Day
        ),
        2023: (
            (JAN, 26, JAN, 28),  # Spring Festival
            (JAN, 27, JAN, 29),  # Spring Festival
            (MAY, 2, APR, 23),  # Labor Day
            (MAY, 3, MAY, 6),  # Labor Day
            (JUN, 23, JUN, 25),  # Dragon Boat Festival
            (OCT, 5, OCT, 7),  # National Day
            (OCT, 6, OCT, 8),  # National Day
        ),
        2024: (
            (FEB, 15, FEB, 4),  # Spring Festival
            (FEB, 16, FEB, 18),  # Spring Festival
            (APR, 5, APR, 7),  # Tomb-Sweeping Day
            (MAY, 2, APR, 28),  # Labor Day
            (MAY, 3, MAY, 11),  # Labor Day
            (SEP, 16, SEP, 14),  # Mid-Autumn Festival
            (OCT, 4, SEP, 29),  # National Day
            (OCT, 7, OCT, 12),  # National Day
        ),
        2025: (
            (FEB, 3, JAN, 26),  # Spring Festival
            (FEB, 4, FEB, 8),  # Spring Festival
            (MAY, 5, APR, 27),  # Labor Day
            (OCT, 7, SEP, 28),  # National Day
            (OCT, 8, OCT, 11),  # National Day
        ),
        2026: (
            (JAN, 2, JAN, 4),  # New Year's Day
            (FEB, 20, FEB, 14),  # Spring Festival
            (FEB, 23, FEB, 28),  # Spring Festival
            (MAY, 5, MAY, 9),  # Labor Day
            (OCT, 6, SEP, 20),  # National Day
            (OCT, 7, OCT, 10),  # National Day
        ),
    }

    special_public_holidays_observed = {
        2012: (JUN, 22, dragon_boat_festival),  # observed from Jun 23
        2015: (OCT, 6, mid_autumn_festival),  # observed from Sep 27
        2020: (
            (JAN, 30, chinese_new_year),  # Spring Festival (extended due to Covid-19 decree)
            (OCT, 6, mid_autumn_festival),  # observed from Oct 1, overlap with National Day
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JUN, JUL, AUG, SEP, OCT, NOV
from holidays.groups import InternationalHolidays, IslamicHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_WORKDAY


class Turkmenistan(ObservedHolidayBase, InternationalHolidays, IslamicHolidays):
    """Turkmenistan holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Turkmenistan>
        * [Labor Code](https://web.archive.org/web/20250419204037/https://www.tds.gov.tm/en/20-labor-code-of-turkmenistan)
        * [State Flag and Constitution Day](https://en.wikipedia.org/wiki/State_Flag_and_Constitution_Day_(Turkmenistan))
        * [Independence Day](https://en.wikipedia.org/wiki/Independence_Day_(Turkmenistan))
        * [Detailed Research](https://archive.org/details/holiday-research-in-central-asia)
        * [Law of 2008-08-15 on Amendments to Labor Code](https://web.archive.org/web/20250710021455/https://www.parahat.info/law/2008-08-16-zakon-turkmenistana-o-vnesenii-izmeneniya-v-kodeks-zakonov-o-trude-turkmenistana)
        * [Law of 2014-10-05 on Amendments to Labor Code](https://web.archive.org/web/20250615083107/https://www.parahat.info/law/2014-10-06-zakon-turkmenistana-o-vnesenii-izmeneniya-v-trudovoy-kodeks-turkmenistana)
        * [Law of 2017-10-11 on Amendments to Labor Code](https://web.archive.org/web/20250623221410/https://www.parahat.info/law/parahat-info-law-02ai)
    """

    country = "TM"
    default_language = "tk"
    # %s (estimated).
    estimated_label = tr("%s (çak edilýär)")
    # %s (observed).
    observed_label = tr("%s (dynç güni)")
    # %s (observed, estimated)
    observed_estimated_label = tr("%s (dynç güni, çak edilýär)")
    start_year = 1992
    supported_languages = ("en_US", "ru", "tk")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=TurkmenistanIslamicHolidays, show_estimated=islamic_show_estimated
        )
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_WORKDAY)
        kwargs.setdefault("observed_since", 2010)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        dts_observed.add(self._add_new_years_day(tr("Täze ýyl")))

        if 1995 <= self._year <= 2017:
            # State Flag Day.
            dts_observed.add(self._add_holiday_feb_19(tr("Türkmenistanyň Döwlet baýdagynyň güni")))

        # Spring Festival.
        name = tr("Milli bahar baýramy")
        if 2001 <= self._year <= 2007:
            dts_observed.add(self._add_holiday_mar_20(name))
        else:
            # International Women's Day.
            dts_observed.add(self._add_womens_day(tr("Halkara zenanlar güni")))

        dts_observed.add(self._add_holiday_mar_21(name))
        dts_observed.add(self._add_holiday_mar_22(name))

        if self._year <= 2017:
            dts_observed.add(
                self._add_holiday_may_9(
                    # Victory Day.
                    tr("1941-1945-nji ýyllaryň Beýik Watançylyk urşunda ýeňiş güni")
                )
            )

        if self._year >= 2009:
            if self._year >= 2018:
                # Constitution and State Flag Day.
                name = tr("Türkmenistanyň Konstitusiýasynyň we Döwlet baýdagynyň güni")
            elif self._year >= 2015:
                # Day of the Constitution of Turkmenistan and Poetry of Magtymguly Pyragy.
                name = tr(
                    "Türkmenistanyň Konstitusiýasynyň we Makhtumkuli Pyragynyň şygryýet güni"
                )
            else:
                # Day of Revival, Unity and Poetry of Magtymguly Pyragy.
                name = tr("Galkynyş, Agzybirlik we Magtymguly Pyragynyň şygryýet güni")
            dts_observed.add(self._add_holiday_may_18(name))

        # Independence Day.
        name = tr("Türkmenistanyň Garaşsyzlyk güni")
        if self._year <= 2017:
            dts_observed.add(self._add_holiday_oct_27(name))
            if self._year >= 2008:
                dts_observed.add(self._add_holiday_oct_28(name))
        else:
            dts_observed.add(self._add_holiday_sep_27(name))

        if self._year >= 1995:
            # Memorial Day.
            name = tr("Hatyra güni")
            if 2009 <= self._year <= 2014:
                dts_observed.add(self._add_holiday_jan_12(name))
                # National Memorial Day.
                dts_observed.add(self._add_holiday_oct_6(tr("Milli ýatlama güni")))
            else:
                dts_observed.add(self._add_holiday_oct_6(name))

            # International Neutrality Day.
            dts_observed.add(self._add_holiday_dec_12(tr("Halkara Bitaraplyk güni")))

        # Eid al-Fitr.
        dts_observed.update(self._add_eid_al_fitr_day(tr("Oraza baýramy")))

        # Eid al-Adha.
        dts_observed.update(self._add_eid_al_adha_day(tr("Gurban baýramy")))

        self._populate_observed(dts_observed)


class TM(Turkmenistan):
    pass


class TKM(Turkmenistan):
    pass


class TurkmenistanIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20240908061230/https://www.timeanddate.com/holidays/turkmenistan/eid-al-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2010, 2025)
    EID_AL_FITR_DATES = {
        2011: (AUG, 31),
        2014: (JUL, 29),
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
    }

    # https://web.archive.org/web/20240912191844/https://www.timeanddate.com/holidays/turkmenistan/eid-al-adha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2010, 2025)
    EID_AL_ADHA_DATES = {
        2010: (NOV, 17),
        2011: (NOV, 7),
        2014: (OCT, 5),
        2015: (SEP, 24),
        2016: (SEP, 13),
        2017: (SEP, 2),
        2018: (AUG, 22),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars.gregorian import SEP, THU, FRI, SAT
from holidays.groups import InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Kuwait(HolidayBase, InternationalHolidays, IslamicHolidays):
    """Kuwait holidays.

    References:
        * <https://en.wikipedia.org/wiki/2024_in_Kuwait>
        * <https://web.archive.org/web/20250414072705/https://www.officeholidays.com/countries/kuwait>
        * <https://web.archive.org/web/20250414072609/https://www.timeanddate.com/holidays/kuwait/2024>
    """

    country = "KW"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (المقدرة)")
    supported_languages = ("ar", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(self, show_estimated=islamic_show_estimated)
        super().__init__(*args, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        # The resting days are Friday and Saturday since Sep 1, 2007.
        # https://web.archive.org/web/20250414072729/https://www.arabnews.com/node/298933
        return {FRI, SAT} if dt >= date(2007, SEP, 1) else {THU, FRI}

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("رأس السنة الميلادية"))

        # National Day.
        self._add_holiday_feb_25(tr("اليوم الوطني"))

        # Liberation Day.
        self._add_holiday_feb_26(tr("يوم التحرير"))

        # Islamic New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("عيد المولد النبوي"))

        # Isra' and Mi'raj.
        self._add_isra_and_miraj_day(tr("ليلة المعراج"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("عيد الفطر"))
        # Eid al-Fitr Holiday.
        self._add_eid_al_fitr_day_two(tr("عطلة عيد الفطر"))
        self._add_eid_al_fitr_day_three(tr("عطلة عيد الفطر"))

        # Arafat Day.
        self._add_arafah_day(tr("يوم عرفة"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("عيد الأضحى"))
        # Eid al-Adha Holiday.
        self._add_eid_al_adha_day_two(tr("عطلة عيد الأضحى"))
        self._add_eid_al_adha_day_three(tr("عطلة عيد الأضحى"))


class KW(Kuwait):
    pass


class KWT(Kuwait):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from __future__ import annotations

from datetime import date
from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import (
    ChristianHolidays,
    IslamicHolidays,
    InternationalHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_WORKDAY,
    SUN_TO_NEXT_WORKDAY,
)


class Kyrgyzstan(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Kyrgyzstan holidays.

    References:
        * [Law 430-XII from Apr 19, 1991](https://cbd.minjust.gov.kg/4-4972/edition/506328/ru)
        * [Labor Code from Oct 4, 1997](https://cbd.minjust.gov.kg/570/edition/282855/ru)
        * [Law 2 from Jan 5, 2002](https://cbd.minjust.gov.kg/949/edition/283884/ru)
        * [Law 3 from Jan 8, 2002](https://cbd.minjust.gov.kg/950/edition/283886/kg)
        * [Labor Code from Aug 4, 2004](https://cbd.minjust.gov.kg/3-22/edition/13763/ru)
        * [Law 21 from Mar 17, 2008](https://cbd.minjust.gov.kg/202231/edition/409047/kg)
        * [Law 103 from Mar 30, 2009](https://cbd.minjust.gov.kg/203130/edition/479924/kg)
        * [Law 97 from Jul 5, 2012](https://cbd.minjust.gov.kg/203697/edition/454078/kg)
        * [Law 185 from Nov 20, 2012](https://cbd.minjust.gov.kg/203785/edition/454186/kg)
        * [Law 34 from Apr 6, 2016](https://cbd.minjust.gov.kg/111313/edition/708448/kg)
        * [Law 190 from Nov 20, 2017](https://cbd.minjust.gov.kg/111697/edition/854612/kg)
        * [Labor Code from Jan 23, 2025](https://cbd.minjust.gov.kg/3-45/edition/25298/kg)

    Eid al-Adha dates sources:
        * [2006](https://cbd.minjust.gov.kg/7-11881/edition/1141210/kg)
        * [2007](https://cbd.minjust.gov.kg/7-12495/edition/1142100/kg)
        * [2008](https://cbd.minjust.gov.kg/7-13117/edition/1142600/kg)
        * [2009](https://cbd.minjust.gov.kg/7-14142/edition/1142824/kg)
        * [2010](https://cbd.minjust.gov.kg/7-14446/edition/1143250/kg)
        * [2011](https://cbd.minjust.gov.kg/7-16708/edition/1143560/kg)
        * [2013](https://cbd.minjust.gov.kg/7-16194/edition/1144562/kg)
        * [2014](https://cbd.minjust.gov.kg/7-17626/edition/1145086/kg)
        * [2015](https://cbd.minjust.gov.kg/7-18423/edition/1145472/kg)
        * [2024](https://cbd.minjust.gov.kg/7-22932/edition/1399/kg)

    Eid al-Fitr dates sources:
        * [2007](https://cbd.minjust.gov.kg/7-12377/edition/1141880/kg)
        * [2008](https://cbd.minjust.gov.kg/7-21129/edition/1142566/kg)
        * [2009](https://cbd.minjust.gov.kg/7-14142/edition/1142824/kg)
        * [2010](https://cbd.minjust.gov.kg/7-14372/edition/1143234/kg)
        * [2011](https://cbd.minjust.gov.kg/7-16615/edition/1143534/kg)
        * [2012](https://cbd.minjust.gov.kg/7-15563/edition/1144054/kg)
        * [2013](https://cbd.minjust.gov.kg/7-16194/edition/1144562/kg)
        * [2014](https://cbd.minjust.gov.kg/7-17399/edition/1144974/kg)
        * [2015](https://cbd.minjust.gov.kg/7-18289/edition/1145412/kg)
        * [2024](https://cbd.minjust.gov.kg/7-22932/edition/1399/kg)

    """

    country = "KG"
    default_language = "ky"
    # %s (estimated).
    estimated_label = tr("%s (болжолдуу)")
    # %s (observed).
    observed_label = tr("%s (көрүлгөн күнү)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (көрүлгөн күнү, болжолдуу)")
    # Kyrgyzstan declared its independence on Aug 31, 1991.
    start_year = 1992
    supported_categories = (PUBLIC, WORKDAY)
    supported_languages = ("en_US", "ky", "ru_KG")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self, JULIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=KyrgyzstanIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, KyrgyzstanStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        super().__init__(*args, **kwargs)

    def _populate_observed(self, dts: set[date], *, multiple: bool = False) -> None:
        # Labor Code from Aug 4, 2004.
        observed_change_start = date(2004, AUG, 4)
        # Law 103 from Mar 30, 2009.
        observed_change_end = date(2009, MAR, 29)
        for dt in sorted(dts):
            # Labor Code from Jan 23, 2025.
            if dt >= date(2025, JAN, 23):
                continue
            self._add_observed(
                dt,
                rule=SUN_TO_NEXT_WORKDAY
                if observed_change_start <= dt <= observed_change_end
                else SAT_SUN_TO_NEXT_WORKDAY,
            )

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        dts_observed.add(jan_1 := self._add_new_years_day(tr("Жаңы жыл")))

        # Established by Labor Code from Jan 23, 2025.
        if self._year >= 2026:
            # New Year Holidays.
            self._add_multiday_holiday(jan_1, 5, name=tr("Жаңы жылдык каникулдар"))

        # Christmas Day.
        dt = self._add_christmas_day(tr("Ыйса Пайгамбардын туулган күнү"))
        # Law 2 from Jan 5, 2002.
        # Law 185 from Nov 20, 2012.
        if self._year <= 2001 or self._year >= 2013:
            dts_observed.add(dt)

        # Established by Labor Code from Aug 4, 2004.
        # Made working by Labor Code from Jan 23, 2025.
        if 2005 <= self._year <= 2024:
            # Fatherland Defender's Day.
            dts_observed.add(self._add_holiday_feb_23(tr("Ата-Журтту коргоонун күнү")))

        # Renamed by Labor Code from Aug 4, 2004.
        dts_observed.add(
            self._add_womens_day(
                # International Women's Day.
                tr("Аялдардын эл аралык күнү")
                if self._year >= 2005
                # Women's Day.
                else tr("Аялдар күнү")
            )
        )

        # Nooruz Holiday.
        dts_observed.add(self._add_holiday_mar_21(tr("Элдик Нооруз майрамы")))

        # Established by Law 21 from Mar 17, 2008.
        # Abolished by Law 97 from Jul 5, 2012.
        if 2008 <= self._year <= 2012:
            # Day of the People's Revolution.
            dts_observed.add(self._add_holiday_mar_24(tr("Элдик революция күнү")))

        # Established by Law 34 from Apr 6, 2016.
        # Made working by Labor Code from Jan 23, 2025.
        if 2016 <= self._year <= 2024:
            # Day of the People's April Revolution.
            dts_observed.add(self._add_holiday_apr_7(tr("Элдик Апрель революциясы күнү")))

        # Renamed by Labor Code from Oct 4, 1997.
        dts_observed.add(
            may_1 := self._add_labor_day(
                # Labor Day.
                tr("Эмгек майрамы")
                if self._year >= 1998
                # International Workers' Solidarity Day.
                else tr("Эмгекчилердин эл аралык тилектештик күнү")
            )
        )

        dts_observed.add(
            # Constitution Day.
            may_5 := self._add_holiday_may_5(tr("Кыргыз Республикасынын Конституция күнү"))
        )

        # Established by Labor Code from Jan 23, 2025.
        if self._year >= 2025:
            # May Holidays.
            name = tr("Май каникулдары")
            self._add_multiday_holiday(may_1, 3, name=name)
            self._add_multiday_holiday(may_5, 3, name=name)

        # Victory Day.
        dts_observed.add(self._add_world_war_two_victory_day(tr("Жеңиш күнү"), is_western=False))

        dts_observed.add(
            # Independence Day.
            self._add_holiday_aug_31(tr("Кыргыз Республикасынын Көз карандысыздыгынын күнү"))
        )

        # Established by Law 190 from Nov 20, 2017.
        # Made working by Labor Code from Jan 23, 2025.
        if self._year <= 2024:
            if self._year >= 2018:
                # Days of History and Commemoration of Ancestors.
                name = tr("Тарых жана ата-бабаларды эскерүү күндөрү")
                dts_observed.add(self._add_holiday_nov_7(name))
                dts_observed.add(self._add_holiday_nov_8(name))

            # Established by Law 3 from Jan 8, 2002.
            elif self._year >= 2002:
                dts_observed.add(
                    # Day of the Great October Socialist Revolution.
                    self._add_holiday_nov_7(tr("Улуу Октябрь социалисттик революциясынын күнү"))
                )

        # Eid al-Fitr.
        dts = self._add_eid_al_fitr_day(tr("Орозо айт"))
        # Law 185 from Nov 20, 2012.
        if self._year >= 2013:
            dts_observed.update(dts)

        # Eid al-Adha.
        dts = self._add_eid_al_adha_day(tr("Курман айт"))
        # Law 185 from Nov 20, 2012.
        if self._year >= 2013:
            dts_observed.update(dts)

        if self.observed:
            self._populate_observed(dts_observed)

    def _populate_workday_holidays(self):
        # Made working by Labor Code from Jan 23, 2025.
        if self._year >= 2025:
            # Fatherland Defender's Day.
            self._add_holiday_feb_23(tr("Мекенди коргоочулардын күнү"))

            # Day of the People's April Revolution.
            self._add_holiday_apr_7(tr("Элдик Апрель революциясы күнү"))

            # Days of History and Commemoration of Ancestors.
            name = tr("Тарых жана ата-бабаларды эскерүү күндөрү")
            self._add_holiday_nov_7(name)
            self._add_holiday_nov_8(name)


class KG(Kyrgyzstan):
    pass


class KGZ(Kyrgyzstan):
    pass


class KyrgyzstanIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2006, 2026)
    EID_AL_ADHA_DATES = {
        2007: (DEC, 19),
        2015: (SEP, 24),
        2016: (SEP, 12),
        2024: (JUN, 17),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2006, 2026)
    EID_AL_FITR_DATES = {
        2006: (OCT, 24),
        2007: (OCT, 12),
        2014: (JUL, 29),
        2016: (JUL, 5),
        2017: (JUN, 26),
        2019: (JUN, 5),
        2022: (MAY, 3),
    }


class KyrgyzstanStaticHolidays:
    """Kyrgyzstan special holidays.

    Substituted holidays references:
        * [2006](https://cbd.minjust.gov.kg/7-11117/edition/1149816/kg)
        * [2007](https://cbd.minjust.gov.kg/7-11881/edition/1141210/kg)
        * [2008](https://cbd.minjust.gov.kg/7-12535/edition/1142124/kg)
        * 2009:
            [1](https://cbd.minjust.gov.kg/7-13161/edition/1142616/kg)
            [2](https://cbd.minjust.gov.kg/7-13259/edition/1142704/kg)
            [3](https://cbd.minjust.gov.kg/7-13214/edition/1142732/kg)
        * 2010:
            [1](https://cbd.minjust.gov.kg/7-13990/edition/1265189/kg)
            [2](https://cbd.minjust.gov.kg/7-14217/edition/1143104/kg)
            [3](https://cbd.minjust.gov.kg/7-14352/edition/1143202/kg)
            [4](https://cbd.minjust.gov.kg/7-14446/edition/1143250/kg)
        * 2011:
            [1](https://cbd.minjust.gov.kg/7-15768/edition/1143374/kg)
            [2](https://cbd.minjust.gov.kg/7-16497/edition/1143464/kg)
            [Nov 8 observed](https://cbd.minjust.gov.kg/7-16708/edition/1143560/kg)
        * 2012:
            [1](https://cbd.minjust.gov.kg/7-14904/edition/1143596/kg)
            [Aug 20 observed](https://cbd.minjust.gov.kg/7-15563/edition/1144054/kg)
        * 2013:
            [1](https://cbd.minjust.gov.kg/7-15246/edition/407825/kg)
            [2](https://cbd.minjust.gov.kg/7-15935/edition/1144342/kg)
            [3](https://cbd.minjust.gov.kg/7-16194/edition/1144562/kg)
            [4](https://cbd.minjust.gov.kg/7-16285/edition/1144690/kg)
        * 2014:
            [1](https://cbd.minjust.gov.kg/7-16375/edition/1144728/kg)
            [2](https://cbd.minjust.gov.kg/7-17399/edition/1144974/kg)
            [3](https://cbd.minjust.gov.kg/7-17534/edition/1145074/kg)
        * [2015](https://cbd.minjust.gov.kg/7-17788/edition/1145282/kg)
        * 2016
            [1](https://cbd.minjust.gov.kg/7-18618/edition/1145496/kg)
            [2](https://cbd.minjust.gov.kg/7-18753/edition/19311/kg)
        * 2017:
            [1](https://cbd.minjust.gov.kg/7-19609/edition/1145794/kg)
            [2](https://cbd.minjust.gov.kg/7-19686/edition/1145808/kg)
            [3](https://cbd.minjust.gov.kg/7-1951/edition/1145958/kg)
        * 2018:
            [1](https://cbd.minjust.gov.kg/7-1813/edition/1145978/kg)
            [2](https://cbd.minjust.gov.kg/7-2294/edition/1146017/kg)
            [3](https://cbd.minjust.gov.kg/7-2354/edition/1146046/kg)
        * [2019](https://cbd.minjust.gov.kg/7-2573/edition/1146108/kg)
        * [2020](https://cbd.minjust.gov.kg/7-20233/edition/1146122/kg)
        * 2021:
            [1](https://cbd.minjust.gov.kg/7-20440/edition/1146148/kg)
            [2](https://cbd.minjust.gov.kg/7-20890/edition/12112/kg)
            [3](https://cbd.minjust.gov.kg/7-20980/edition/12120/kg)
        * 2022:
            [1](https://cbd.minjust.gov.kg/7-21274/edition/12170/kg)
            [2](https://cbd.minjust.gov.kg/7-21501/edition/12246/kg)
        * [2023](https://cbd.minjust.gov.kg/7-21921/edition/1252593/kg)
        * [2024](https://cbd.minjust.gov.kg/7-22932/edition/1399/kg)
        * [2025](https://cbd.minjust.gov.kg/7-36549/edition/26567/kg)
        * [2025 changes](https://cbd.minjust.gov.kg/7-38206/edition/26566/kg)
    """

    # Substituted date format.
    substituted_date_format = tr("%d.%m.%Y")

    # Day off (substituted from %s).
    substituted_label = tr("Эс алуу күнү (%s күнүнөн которулган)")

    # Eid al-Adha.
    eid_al_adha = tr("Курман айт")

    # Additional public holiday.
    additional_public_holiday = tr("Кошумча эс алуу күнү")

    special_public_holidays = {
        2006: (JAN, 9, JAN, 15),
        2007: (
            (JAN, 3, DEC, 30, 2006),
            (JAN, 4, JAN, 13),
        ),
        2008: (
            (JAN, 2, DEC, 29, 2007),
            (JAN, 3, DEC, 30, 2007),
            (JAN, 4, JAN, 12),
        ),
        2009: (
            (JAN, 2, DEC, 27, 2008),
            (JAN, 5, JAN, 10),
            (JAN, 6, JAN, 17),
            (MAR, 23, MAR, 28),
            (MAY, 4, MAY, 16),
            (DEC, 31, DEC, 26),
        ),
        2010: (
            (JAN, 4, JAN, 16),
            (JAN, 5, JAN, 23),
            (JAN, 6, JAN, 30),
            (JAN, 8, FEB, 6),
            (FEB, 22, FEB, 20),
            (AUG, 30, AUG, 28),
            (NOV, 15, NOV, 13),
        ),
        2011: (
            (MAR, 7, MAR, 5),
            (MAY, 6, MAY, 14),
        ),
        2012: (
            (JAN, 3, JAN, 14),
            (JAN, 4, JAN, 21),
            (JAN, 5, JAN, 28),
            (JAN, 6, FEB, 4),
        ),
        2013: (
            (JAN, 2, DEC, 29, 2012),
            (JAN, 3, JAN, 12),
            (JAN, 4, JAN, 19),
            (MAY, 10, MAY, 6),
            (AUG, 9, AUG, 17),
            (OCT, 14, OCT, 12),
            (NOV, 8, NOV, 16),
        ),
        2014: (
            (JAN, 2, JAN, 11),
            (JAN, 3, JAN, 18),
            (JAN, 6, JAN, 25),
            (JUL, 28, AUG, 2),
            (SEP, 5, SEP, 1),
        ),
        2015: (
            (JAN, 2, JAN, 10),
            (JAN, 5, FEB, 21),
            (JAN, 6, MAR, 23),
        ),
        2016: (
            (JAN, 8, JAN, 16),
            (MAY, 3, APR, 30),
            (MAY, 4, MAY, 7),
        ),
        2017: (
            (MAR, 20, MAR, 25),
            (MAY, 8, MAY, 13),
            (NOV, 6, OCT, 28),
        ),
        2018: (
            (JAN, 2, DEC, 30, 2017),
            (JAN, 3, JAN, 8),
            (MAR, 9, MAR, 3),
            (APR, 30, APR, 28),
            (AUG, 20, SEP, 22),
            (SEP, 3, MAY, 7),
            (NOV, 9, NOV, 3),
        ),
        2019: (
            (JAN, 2, FEB, 25),
            (JAN, 3, APR, 8),
            (JAN, 4, SEP, 2),
            (MAY, 10, MAY, 6),
        ),
        2020: (
            (JAN, 2, FEB, 24),
            (JAN, 3, MAR, 9),
            (JAN, 6, MAR, 23),
            (MAY, 4, MAY, 11),
        ),
        2021: (
            (JAN, 8, MAY, 3),
            (AUG, 30, MAY, 10),
            (JUL, 19, JUL, 24),
        ),
        2022: (
            (JAN, 4, JAN, 15),
            (JAN, 5, FEB, 26),
            (JAN, 6, MAR, 26),
            (MAY, 4, APR, 30),
            (MAY, 6, MAY, 14),
        ),
        2023: (
            (JAN, 3, JAN, 9),
            (JAN, 4, FEB, 25),
            (JAN, 5, MAR, 11),
            (JAN, 6, MAR, 25),
            (MAR, 20, APR, 29),
            (MAY, 8, MAY, 13),
            (NOV, 6, NOV, 11),
        ),
        2024: (
            (JAN, 2, JAN, 8),
            (JAN, 3, JAN, 27),
            (JAN, 4, MAR, 2),
            (JAN, 5, SEP, 2),
            (MAR, 22, APR, 8),
            (MAY, 2, MAY, 6),
            (MAY, 3, MAY, 11),
        ),
        2025: (
            (JAN, 2, JAN, 11),
            (JAN, 3, additional_public_holiday),
            (JAN, 6, additional_public_holiday),
        ),
    }

    special_public_holidays_observed = {
        2007: (
            (JAN, 2, eid_al_adha),
            # Christmas Day.
            (JAN, 5, tr("Ыйса Пайгамбардын туулган күнү")),
        ),
        2011: (NOV, 8, eid_al_adha),
        # Eid al-Fitr.
        2012: (AUG, 20, tr("Орозо айт")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JUN, SEP, NOV, THU, FRI, SAT, _timedelta
from holidays.groups import IslamicHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    THU_TO_PREV_WED,
    FRI_TO_PREV_THU,
    FRI_TO_NEXT_SAT,
    SAT_TO_NEXT_SUN,
    THU_FRI_TO_NEXT_WORKDAY,
    FRI_SAT_TO_NEXT_WORKDAY,
)


class SaudiArabia(ObservedHolidayBase, IslamicHolidays, StaticHolidays):
    """Saudi Arabia holidays.

    References:
        * <https://ar.wikipedia.org/wiki/قائمة_العطل_الرسمية_في_السعودية>
        * <https://web.archive.org/web/20240610223551/http://laboreducation.hrsd.gov.sa/en/labor-education/322>
        * <https://web.archive.org/web/20250329052253/https://english.alarabiya.net/News/gulf/2022/01/27/Saudi-Arabia-to-commemorate-Founding-Day-on-Feb-22-annually-Royal-order>
        * [2015 (1436 AH) Dhu al-Hijjah begin on September 15](https://web.archive.org/web/20250430191246/https://qna.org.qa/en/news/news-details?id=saudi-arabia-eid-aladha-to-start-on-september-24&date=14/09/2015)
    """

    country = "SA"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (المقدرة)")
    # %s (observed).
    observed_label = tr("%s (ملاحظة)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (المقدرة، ملاحظة)")
    supported_languages = ("ar", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = False, **kwargs):
        """Saudi Arabia has traditionally used the Umm al-Qura calendar
            for administrative purposes.

        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        IslamicHolidays.__init__(
            self, cls=SaudiArabiaIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, SaudiArabiaStaticHolidays)
        kwargs.setdefault("observed_rule", FRI_TO_PREV_THU + SAT_TO_NEXT_SUN)
        super().__init__(*args, **kwargs)

    def _add_islamic_observed(self, dt: date) -> None:
        # Observed days are added to make up for any days falling on a weekend.
        if not self.observed:
            return None
        observed_rule = THU_FRI_TO_NEXT_WORKDAY if self._year <= 2012 else FRI_SAT_TO_NEXT_WORKDAY
        for i in range(4):
            self._add_observed(_timedelta(dt, -i), name=self[dt], rule=observed_rule)

    def _get_weekend(self, dt: date) -> set[int]:
        # Weekend used to be THU, FRI before June 28th, 2013.
        return {FRI, SAT} if dt >= date(2013, JUN, 28) else {THU, FRI}

    def _populate_public_holidays(self):
        self._observed_rule = (
            THU_TO_PREV_WED + FRI_TO_NEXT_SAT
            if self._year <= 2012
            else FRI_TO_PREV_THU + SAT_TO_NEXT_SUN
        )

        # Eid al-Fitr Holiday.
        eid_al_fitr_name = tr("عطلة عيد الفطر")
        eid_al_fitr_dates = self._add_eid_al_fitr_day(eid_al_fitr_name)
        self._add_eid_al_fitr_day_two(eid_al_fitr_name)
        self._add_eid_al_fitr_day_three(eid_al_fitr_name)

        for dt in eid_al_fitr_dates:
            if self._islamic_calendar._is_long_ramadan(dt):
                # Add 30 Ramadan.
                self._add_holiday(eid_al_fitr_name, _timedelta(dt, -1))
                self._add_islamic_observed(_timedelta(dt, +2))
            else:
                # Add 4 Shawwal.
                self._add_islamic_observed(self._add_holiday(eid_al_fitr_name, _timedelta(dt, +3)))

        # Arafat Day.
        self._add_arafah_day(tr("يوم عرفة"))

        # Eid al-Adha Holiday.
        name = tr("عطلة عيد الأضحى")
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        for dt in self._add_eid_al_adha_day_three(name):
            self._add_islamic_observed(dt)

        if self._year >= 2022:
            # Founding Day.
            self._add_observed(self._add_holiday_feb_22(tr("يوم التأسيسي")))

        if self._year >= 2005:
            # National Day.
            self._add_observed(self._add_holiday_sep_23(tr("اليوم الوطني")))


class SA(SaudiArabia):
    pass


class SAU(SaudiArabia):
    pass


class SaudiArabiaIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_ADHA_DATES = {
        2015: (SEP, 24),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2001, 2025)


class SaudiArabiaStaticHolidays:
    special_public_holidays = {
        # Celebrate the country's win against Argentina in the World Cup.
        2022: (NOV, 23, tr("يوم وطني")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from __future__ import annotations

from gettext import gettext as tr
from typing import TYPE_CHECKING

from holidays.calendars.gregorian import THU, FRI, SAT, SUN
from holidays.constants import CHRISTIAN, HEBREW, PUBLIC
from holidays.groups import (
    ChristianHolidays,
    HebrewCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
)
from holidays.holiday_base import HolidayBase

if TYPE_CHECKING:
    from datetime import date


class Algeria(
    HolidayBase, ChristianHolidays, HebrewCalendarHolidays, InternationalHolidays, IslamicHolidays
):
    """Algeria holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Algeria>
        * [Loi n° 63-278 du 26 juillet 1963](https://web.archive.org/web/20250903142053/https://drive.google.com/file/d/1rWkxBEjs9aNkbiTTRXBET3Qo_3HQOJR9/view)
        * [Ordonnance n° 68-419 du 26 juin 1968](https://web.archive.org/web/20250903142828/https://drive.google.com/file/d/1fMGyGVnunGkACWhm0tICANEZqao2pOV3/view)
        * [Loi n° 05-06 du 17 Rabie El Aoeul 1426](https://web.archive.org/web/20250903143453/https://drive.google.com/file/d/1AXISrPGJzgDO8G3uB1TyICdtnOYW6ewm/view)
        * [Loi n° 18-12 du 18 Chaoual 1439](https://web.archive.org/web/20250903141752/https://drive.google.com/file/d/1qxcrF3J-SUXmLdGZLVF0gdrN0aQWwykb/view)
        * [Loi n° 23-10 du 8 Dhou El Hidja 1444](https://web.archive.org/web/20250903141052/https://drive.google.com/file/d/1hZJtxofzjFAphOX09dGGtvHxl6IN5aNz/view)
        * <https://web.archive.org/web/20241231091630/https://www.thenationalnews.com/mena/2021/12/07/when-is-the-weekend-in-the-arab-world/>
        * <https://web.archive.org/web/20250903160618/https://www.lemonde.fr/afrique/article/2018/01/12/en-algerie-le-nouvel-an-berbere-ferie-pour-la-premiere-fois_5241028_3212.html>
    """

    country = "DZ"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (المقدرة)")
    # Loi n° 63-278 du 26 juillet 1963.
    start_year = 1964
    supported_categories = (CHRISTIAN, HEBREW, PUBLIC)
    supported_languages = ("ar", "en_US", "fr")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        HebrewCalendarHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(self, show_estimated=islamic_show_estimated)
        super().__init__(*args, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        # The resting days are Friday and Saturday since 2009.
        # Previously, these were on Thursday and Friday as implemented in 1976.
        if dt.year >= 2009:
            return {FRI, SAT}
        elif dt.year >= 1976:
            return {THU, FRI}
        else:
            return {SAT, SUN}

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("رأس السنة الميلادية"))

        # First Celebrated in 2018.
        # Reaffirmed via Loi n° 18-12 du 18 Chaoual 1439.
        if self._year >= 2018:
            # Amazigh New Year.
            self._add_holiday_jan_12(tr("رأس السنة الأمازيغية"))

        # Labor Day.
        self._add_labor_day(tr("عيد العمال"))

        # Name changed in Loi n° 05-06 du 17 Rabie El Aoeul 1426.
        self._add_holiday_jul_5(
            # Independence Day.
            tr("عيد الاستقلال")
            if self._year >= 2005
            # Independence and National Liberation Front Day.
            else tr("عيد الاستقلال وجبهة التحرير الوطني")
        )

        # Revolution Day.
        self._add_holiday_nov_1(tr("عيد الثورة"))

        # Islamic New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        # Ashura.
        self._add_ashura_day(tr("عاشورة"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("عيد المولد النبوي"))

        # Eid al-Fitr.
        name = tr("عيد الفطر")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        # Added via Loi n° 23-10 du 8 Dhou El Hidja 1444.
        if self._year >= 2024:
            self._add_eid_al_fitr_day_three(name)

        # Eid al-Adha.
        name = tr("عيد الأضحى")
        self._add_eid_al_adha_day(name)
        # Added via Ordonnance n° 68-419 du 26 juin 1968.
        if self._year >= 1969:
            self._add_eid_al_adha_day_two(name)
            # Added via Loi n° 23-10 du 8 Dhou El Hidja 1444.
            if self._year >= 2023:
                self._add_eid_al_adha_day_three(name)

    def _populate_christian_holidays(self):
        # As outlined in Loi n° 63-278 du 26 juillet 1963.

        # Easter Monday.
        self._add_easter_monday(tr("إثنين الفصح"))

        # Ascension Day.
        self._add_ascension_thursday(tr("عيد الصعود"))

        # Whit Monday.
        self._add_whit_monday(tr("إثنين العنصرة"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("عيد انتقال السيدة العذراء"))

        # Christmas Day.
        self._add_christmas_day(tr("عيد الميلاد"))

    def _populate_hebrew_holidays(self):
        # As outlined in Loi n° 63-278 du 26 juillet 1963.

        # Rosh Hashanah.
        self._add_rosh_hashanah(tr("رأس السنة العبرية"))

        # Yom Kippur.
        self._add_yom_kippur(tr("يوم الغفران"))

        # Pesach.
        self._add_passover(tr("عيد الفصح اليهودي"))


class DZ(Algeria):
    pass


class DZA(Algeria):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.julian_revised import JULIAN_REVISED_CALENDAR
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Romania(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Romania holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Romania>
        * <https://web.archive.org/web/20250414065123/https://www.dreptonline.ro/legislatie/codul_muncii.php>
    """

    country = "RO"
    default_language = "ro"
    supported_languages = ("en_US", "ro", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self, JULIAN_REVISED_CALENDAR)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = tr("Anul Nou")
        self._add_new_years_day(name)
        self._add_new_years_day_two(name)

        if self._year >= 2024:
            # Epiphany.
            self._add_epiphany_day(tr("Bobotează"))

            # Saint John the Baptist.
            self._add_holiday_jan_7(tr("Sfântul Ion"))

        if self._year >= 2016:
            # Unification of the Romanian Principalities Day.
            self._add_holiday_jan_24(tr("Ziua Unirii Principatelor Române"))

        # Easter.
        name = tr("Paștele")
        if self._year >= 2018:
            self._add_good_friday(name)

        self._add_easter_sunday(name)
        self._add_easter_monday(name)

        # Labor Day.
        self._add_labor_day(tr("Ziua Muncii"))

        if self._year >= 2017:
            # Children's Day.
            self._add_childrens_day(tr("Ziua Copilului"))

        # Pentecost.
        name = tr("Rusaliile")
        self._add_whit_sunday(name)
        self._add_whit_monday(name)

        # Law #202/2008
        if self._year >= 2009:
            # Dormition of the Mother of God.
            self._add_assumption_of_mary_day(tr("Adormirea Maicii Domnului"))

        # Law #147/2012
        if self._year >= 2012:
            # Saint Andrew's Day.
            self._add_holiday_nov_30(tr("Sfantul Apostol Andrei cel Intai chemat"))

        # National Day.
        self._add_holiday_dec_1(tr("Ziua Națională a României"))

        # Christmas Day.
        name = tr("Crăciunul")
        self._add_christmas_day(name)
        self._add_christmas_day_two(name)


class RO(Romania):
    pass


class ROU(Romania):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, NOV, DEC
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase


class Russia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Russia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Russia>
        * <https://ru.wikipedia.org/wiki/Праздники_России>
    """

    country = "RU"
    default_language = "ru"
    # %s (observed).
    observed_label = tr("%s (выходной)")
    supported_languages = ("en_US", "ru", "th")
    start_year = 1991

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self, JULIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, RussiaStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        if self._year <= 2004:
            # New Year's Day.
            name = tr("Новый год")
            self._add_new_years_day(name)
            if self._year >= 1993:
                self._add_new_years_day_two(name)
        else:
            # New Year Holidays.
            name = tr("Новогодние каникулы")
            self._add_multiday_holiday(self._add_new_years_day(name), 4)
            if self._year >= 2013:
                self._add_holiday_jan_6(name)
                self._add_holiday_jan_8(name)

        # Christmas Day.
        self._add_christmas_day(tr("Рождество Христово"))

        if self._year >= 2002:
            # Defender of the Fatherland Day.
            self._add_holiday_feb_23(tr("День защитника Отечества"))

        # International Women's Day.
        self._add_womens_day(tr("Международный женский день"))

        name = (
            # Holiday of Spring and Labor.
            tr("Праздник Весны и Труда")
            if self._year >= 1992
            # International Workers' Solidarity Day.
            else tr("День международной солидарности трудящихся")
        )
        self._add_labor_day(name)
        if self._year <= 2004:
            self._add_labor_day_two(name)

        # Victory Day.
        self._add_world_war_two_victory_day(tr("День Победы"), is_western=False)

        if self._year >= 1992:
            self._add_holiday_jun_12(
                # Russia Day.
                tr("День России")
                if self._year >= 2002
                # Day of the Adoption of the Declaration of Sovereignty of the Russian Federation.
                else tr(
                    "День принятия Декларации о государственном суверенитете Российской Федерации"
                )
            )

        if self._year >= 2005:
            # Unity Day.
            self._add_holiday_nov_4(tr("День народного единства"))

        if self._year <= 2004:
            name = (
                # Day of consent and reconciliation.
                tr("День согласия и примирения")
                if self._year >= 1996
                # Anniversary of the Great October Socialist Revolution.
                else tr("Годовщина Великой Октябрьской социалистической революции")
            )
            self._add_holiday_nov_7(name)
            if self._year <= 1991:
                self._add_holiday_nov_8(name)

        if 1994 <= self._year <= 2004:
            # Constitution Day.
            self._add_holiday_dec_12(tr("День Конституции Российской Федерации"))


class RU(Russia):
    pass


class RUS(Russia):
    pass


class RussiaStaticHolidays:
    # Date format (see strftime() Format Codes).
    substituted_date_format = tr("%d.%m.%Y")
    # Day off (substituted from %s).
    substituted_label = tr("Выходной (перенесено с %s)")

    new_years_day = tr("Новый год")

    new_year_holidays = tr("Новогодние каникулы")

    christmas_day = tr("Рождество Христово")

    defender_of_fatherland_day = tr("День защитника Отечества")

    international_womens_day = tr("Международный женский день")

    labor_day = tr("Праздник Весны и Труда")

    victory_day = tr("День Победы")

    declaration_of_sovereignty_day = tr(
        "День принятия Декларации о государственном суверенитете Российской Федерации"
    )

    russia_day = tr("День России")

    unity_day = tr("День народного единства")

    great_october_socialist_revolution_day = tr(
        "Годовщина Великой Октябрьской социалистической революции"
    )

    consent_and_reconciliation_day = tr("День согласия и примирения")

    constitution_day = tr("День Конституции Российской Федерации")

    special_public_holidays = {
        # Substituted Holidays 1991
        # src: https://ru.wikipedia.org/wiki/Праздники_России
        1991: (
            (MAY, 3, MAY, 5),
            (MAY, 10, MAY, 12),
        ),
        # Substituted Holidays 1994
        # src: https://web.archive.org/web/20250414071308/https://www.consultant.ru/document/cons_doc_LAW_3235/
        1994: (MAR, 7, MAR, 5),
        # Substituted Holidays 1995
        # src: https://web.archive.org/web/20250414071446/https://www.consultant.ru/document/cons_doc_LAW_6316/
        #      https://web.archive.org/web/20250414071453/https://www.consultant.ru/document/cons_doc_LAW_8134/
        #      https://web.archive.org/web/20250414071503/https://www.consultant.ru/document/cons_doc_LAW_8499/
        1995: (
            (MAY, 8, MAY, 6),
            (NOV, 6, NOV, 4),
            (DEC, 11, DEC, 9),
        ),
        # Substituted Holidays 1996
        # src: https://ru.wikipedia.org/wiki/Праздники_России
        1996: (
            (MAY, 3, MAY, 5),
            (MAY, 10, MAY, 12),
            (JUL, 3, DEC, 14),
            (NOV, 8, NOV, 10),
            (DEC, 13, DEC, 15),
        ),
        # Substituted Holidays 1997
        # src: https://ru.wikipedia.org/wiki/Праздники_России
        1997: (
            (JAN, 3, JAN, 5),
            (JUN, 13, JUN, 15),
        ),
        # Substituted Holidays 1999
        # src: https://ru.wikipedia.org/wiki/Праздники_России
        1999: (JAN, 8, JAN, 10),
        # Substituted Holidays 2000
        # src: https://web.archive.org/web/20250414071351/https://www.consultant.ru/document/cons_doc_LAW_25401/
        2000: (
            (MAY, 8, MAY, 6),
            (NOV, 6, NOV, 4),
            (DEC, 11, DEC, 9),
        ),
        # Substituted Holidays 2001
        # src: https://ru.wikipedia.org/wiki/Праздники_России
        2001: (
            (MAR, 9, MAR, 11),
            (APR, 30, APR, 28),
            (JUN, 11, JUN, 9),
            (DEC, 31, DEC, 29),
        ),
        # Substituted Holidays 2002
        # src: https://web.archive.org/web/20240919074957/https://www.consultant.ru/document/cons_doc_LAW_33943/
        2002: (
            (MAY, 3, APR, 27),
            (MAY, 10, MAY, 18),
            (NOV, 8, NOV, 10),
            (DEC, 13, DEC, 15),
        ),
        # Substituted Holidays 2003
        # src: https://web.archive.org/web/20190220082540/http://www.consultant.ru:80/document/cons_doc_LAW_39997/16abd0212dcc02f64a1daed24286d2ffc2a4a9e9
        2003: (
            (JAN, 3, JAN, 4),
            (JAN, 6, JAN, 5),
            (JUN, 13, JUN, 21),
        ),
        # Substituted Holidays 2005
        # src: https://web.archive.org/web/20250414071549/https://www.consultant.ru/document/cons_doc_LAW_50948/
        #      https://ru.wikipedia.org/wiki/Праздники_России
        2005: (
            (MAR, 7, MAR, 5),
            (MAY, 10, MAY, 14),
        ),
        # Substituted Holidays 2006
        # src: https://web.archive.org/web/20250418072310/https://www.consultant.ru/document/cons_doc_LAW_57425/
        2006: (
            (FEB, 24, FEB, 26),
            (MAY, 8, MAY, 6),
        ),
        # Substituted Holidays 2007
        # src: https://web.archive.org/web/20250414071613/https://www.consultant.ru/document/cons_doc_LAW_63838/
        2007: (
            (APR, 30, APR, 28),
            (JUN, 11, JUN, 9),
            (DEC, 31, DEC, 29),
        ),
        # Substituted Holidays 2008
        # src: https://web.archive.org/web/20250420141031/https://www.consultant.ru/document/cons_doc_LAW_70469/
        2008: (
            (MAY, 2, MAY, 4),
            (JUN, 13, JUN, 7),
            (NOV, 3, NOV, 1),
        ),
        # Substituted Holidays 2009
        # src: https://web.archive.org/web/20250418041918/https://www.consultant.ru/document/cons_doc_LAW_81981/
        2009: (JAN, 9, JAN, 11),
        # Substituted Holidays 2010
        # src: https://web.archive.org/web/20250419192633/https://www.consultant.ru/document/cons_doc_LAW_93374/
        2010: (
            (FEB, 22, FEB, 27),
            (NOV, 5, NOV, 13),
        ),
        # Substituted Holidays 2011
        # src: https://web.archive.org/web/20250416021056/https://www.consultant.ru/document/cons_doc_LAW_103530/
        2011: (MAR, 7, MAR, 5),
        # Substituted Holidays 2012
        # src: https://web.archive.org/web/20250420053252/https://www.consultant.ru/document/cons_doc_LAW_117190/
        2012: (
            (MAR, 9, MAR, 11),
            (APR, 30, APR, 28),
            (MAY, 7, MAY, 5),
            (MAY, 8, MAY, 12),
            (JUN, 11, JUN, 9),
            (DEC, 31, DEC, 29),
        ),
        # Substituted Holidays 2013
        # src: https://web.archive.org/web/20250422040920/https://www.consultant.ru/document/cons_doc_LAW_136654/
        2013: (
            (MAY, 2, JAN, 5),
            (MAY, 3, JAN, 6),
            (MAY, 10, FEB, 23),
        ),
        # Substituted Holidays 2014
        # src: https://web.archive.org/web/20250415234224/https://www.consultant.ru/document/cons_doc_LAW_146983/
        2014: (
            (MAY, 2, JAN, 4),
            (JUN, 13, JAN, 5),
            (NOV, 3, FEB, 23),
        ),
        # Substituted Holidays 2015
        # src: https://web.archive.org/web/20250415134053/https://www.consultant.ru/document/cons_doc_LAW_167928/
        2015: (
            (JAN, 9, JAN, 3),
            (MAY, 4, JAN, 4),
        ),
        # Substituted Holidays 2016
        # src: https://web.archive.org/web/20250417021701/https://www.consultant.ru/document/cons_doc_LAW_186505/
        2016: (
            (FEB, 22, FEB, 20),
            (MAR, 7, JAN, 2),
            (MAY, 3, JAN, 3),
        ),
        # Substituted Holidays 2017
        # src: https://web.archive.org/web/20250414184011/https://www.consultant.ru/document/cons_doc_LAW_202871/
        2017: (
            (FEB, 24, JAN, 1),
            (MAY, 8, JAN, 7),
        ),
        # Substituted Holidays 2018
        # src: https://web.archive.org/web/20250414223644/https://www.consultant.ru/document/cons_doc_LAW_280526/
        2018: (
            (MAR, 9, JAN, 6),
            (APR, 30, APR, 28),
            (MAY, 2, JAN, 7),
            (JUN, 11, JUN, 9),
            (DEC, 31, DEC, 29),
        ),
        # Substituted Holidays 2019
        # src: https://web.archive.org/web/20250414103842/https://www.consultant.ru/document/cons_doc_LAW_307996/
        2019: (
            (MAY, 2, JAN, 5),
            (MAY, 3, JAN, 6),
            (MAY, 10, FEB, 23),
        ),
        # Substituted Holidays 2020
        # src: https://web.archive.org/web/20250414071714/https://www.consultant.ru/document/cons_doc_law_328918/
        2020: (
            (MAY, 4, JAN, 4),
            (MAY, 5, JAN, 5),
        ),
        # Substituted Holidays 2021
        # src: https://web.archive.org/web/20250414071843/https://www.consultant.ru/document/cons_doc_law_365179/
        2021: (
            (NOV, 5, JAN, 2),
            (FEB, 22, FEB, 20),
            (DEC, 31, JAN, 3),
        ),
        # Substituted Holidays 2022
        # src: https://web.archive.org/web/20240613213507/https://www.consultant.ru/document/cons_doc_LAW_395538/
        2022: (
            (MAR, 7, MAR, 5),
            (MAY, 3, JAN, 1),
            (MAY, 10, JAN, 2),
        ),
        # Substituted Holidays 2023
        # src: https://web.archive.org/web/20240908190857/https://www.consultant.ru/document/cons_doc_LAW_425407/
        2023: (
            (FEB, 24, JAN, 1),
            (MAY, 8, JAN, 8),
        ),
        # Substituted Holidays 2024
        # src: https://web.archive.org/web/20250417024116/https://www.consultant.ru/document/cons_doc_LAW_455140/
        2024: (
            (APR, 29, APR, 27),
            (APR, 30, NOV, 2),
            (MAY, 10, JAN, 6),
            (DEC, 30, DEC, 28),
            (DEC, 31, JAN, 7),
        ),
        # Substituted Holidays 2025
        # src: https://web.archive.org/web/20250414071812/https://www.consultant.ru/document/cons_doc_LAW_481586/
        2025: (
            (MAY, 2, JAN, 4),
            (MAY, 8, FEB, 23),
            (JUN, 13, MAR, 8),
            (NOV, 3, NOV, 1),
            (DEC, 31, JAN, 5),
        ),
    }

    # These are cases where additional in-lieus are given
    # without actually making weekend workdays.
    # src: https://ru.wikipedia.org/wiki/Праздники_России
    special_public_holidays_observed = {
        1992: (
            (MAY, 4, labor_day),
            (MAY, 11, victory_day),
            (NOV, 9, great_october_socialist_revolution_day),
        ),
        1993: (
            (JAN, 4, new_years_day),
            (MAY, 3, labor_day),
            (MAY, 4, labor_day),
            (MAY, 10, victory_day),
            (JUN, 14, declaration_of_sovereignty_day),
            (NOV, 8, great_october_socialist_revolution_day),
        ),
        1994: (
            (JAN, 3, new_years_day),
            (JAN, 4, new_years_day),
            (MAY, 3, labor_day),
            (MAY, 10, victory_day),
            (JUN, 13, declaration_of_sovereignty_day),
        ),
        1995: (
            (JAN, 3, new_years_day),
            (JAN, 9, christmas_day),
        ),
        1996: (JAN, 8, christmas_day),
        1997: (MAR, 10, international_womens_day),
        1998: (
            (MAR, 9, international_womens_day),
            (MAY, 4, labor_day),
            (MAY, 11, victory_day),
            (NOV, 9, consent_and_reconciliation_day),
            (DEC, 14, constitution_day),
        ),
        1999: (
            (JAN, 4, new_years_day),
            (MAY, 3, labor_day),
            (MAY, 4, labor_day),
            (MAY, 10, victory_day),
            (JUN, 14, declaration_of_sovereignty_day),
            (NOV, 8, consent_and_reconciliation_day),
            (DEC, 13, constitution_day),
        ),
        2000: (
            (JAN, 3, new_years_day),
            (JAN, 4, new_years_day),
        ),
        2001: (JAN, 8, christmas_day),
        2002: (FEB, 25, defender_of_fatherland_day),
        2003: (
            (FEB, 24, defender_of_fatherland_day),
            (MAR, 10, international_womens_day),
        ),
        2004: (
            (MAY, 3, labor_day),
            (MAY, 4, labor_day),
            (MAY, 10, victory_day),
            (JUN, 14, russia_day),
            (NOV, 8, consent_and_reconciliation_day),
            (DEC, 13, constitution_day),
        ),
        2005: (
            (JAN, 6, new_years_day),
            (JAN, 10, new_years_day),
            (MAY, 2, labor_day),
            (JUN, 13, russia_day),
        ),
        2006: (
            (JAN, 6, new_years_day),
            (JAN, 9, christmas_day),
            (NOV, 6, unity_day),
        ),
        2007: (
            (JAN, 8, christmas_day),
            (NOV, 5, unity_day),
        ),
        2008: (
            (JAN, 8, new_year_holidays),
            (FEB, 25, defender_of_fatherland_day),
            (MAR, 10, international_womens_day),
        ),
        2009: (
            (JAN, 6, new_year_holidays),
            (JAN, 8, new_year_holidays),
            (MAR, 9, international_womens_day),
            (MAY, 11, victory_day),
        ),
        2010: (
            (JAN, 6, new_year_holidays),
            (JAN, 8, new_year_holidays),
            (MAY, 3, labor_day),
            (MAY, 10, victory_day),
            (JUN, 14, russia_day),
        ),
        2011: (
            (JAN, 6, new_year_holidays),
            (JAN, 10, new_year_holidays),
            (MAY, 2, labor_day),
            (JUN, 13, russia_day),
        ),
        2012: (
            (JAN, 6, new_year_holidays),
            (JAN, 9, christmas_day),
            (NOV, 5, unity_day),
        ),
        2015: (
            (MAR, 9, international_womens_day),
            (MAY, 11, victory_day),
        ),
        2016: (
            (MAY, 2, labor_day),
            (JUN, 13, russia_day),
        ),
        2017: (NOV, 6, unity_day),
        2018: (NOV, 5, unity_day),
        2020: (
            (FEB, 24, defender_of_fatherland_day),
            (MAR, 9, international_womens_day),
            (MAY, 11, victory_day),
        ),
        2021: (
            (MAY, 3, labor_day),
            (MAY, 10, victory_day),
            (JUN, 14, russia_day),
        ),
        2022: (
            (MAY, 2, labor_day),
            (JUN, 13, russia_day),
        ),
        2023: (NOV, 6, unity_day),
    }
